@@ -30,7 +30,6 @@
 	that work under terms of your choice, so long as this FAUST
 	architecture section is not modified.
 
-
  ************************************************************************
  ************************************************************************/
 
@@ -38,13 +37,17 @@
 #include <stdlib.h>
 #include <iostream>
 
-#include "gui/FUI.h"
-#include "misc.h"
-#include "gui/faustqt.h"
-#include "audio/netjack-dsp.h"
+#include "faust/gui/FUI.h"
+#include "faust/misc.h"
+#include "faust/gui/faustqt.h"
+#include "faust/audio/netjack-dsp.h"
 
 #ifdef OSCCTRL
-#include "gui/OSCUI.h"
+#include "faust/gui/OSCUI.h"
+#endif
+
+#ifdef HTTPCTRL
+#include "faust/gui/httpdUI.h"
 #endif
 
 /**************************BEGIN USER SECTION **************************/
@@ -59,7 +62,6 @@
 
 <<includeIntrinsic>>
 
-
 <<includeclass>>
 
 /***************************END USER SECTION ***************************/
@@ -68,53 +70,81 @@
 
 mydsp DSP;
 
-list<GUI*> GUI::fGuiList;
+std::list<GUI*> GUI::fGuiList;
+ztimedmap GUI::gTimedZoneMap;
 
 //-------------------------------------------------------------------------
 // 									MAIN
 //-------------------------------------------------------------------------
 int main(int argc, char *argv[])
 {
-	char appname[256];
-	char rcfilename[256];
-	char* home = getenv("HOME");
+    char appname[256];
+    char rcfilename[256];
+    char* home = getenv("HOME");
 
     int	celt = lopt(argv, "--celt", -1);
-    char* master_ip = lopts(argv, "--a", DEFAULT_MULTICAST_IP);
+    const char* master_ip = lopts(argv, "--a", DEFAULT_MULTICAST_IP);
     int master_port = lopt(argv, "--p", DEFAULT_PORT);
+    int mtu = lopt(argv, "--m", DEFAULT_MTU);
+    int latency = lopt(argv, "--l", 2);
 
-	snprintf(appname, 255, "%s", basename(argv[0]));
-	snprintf(rcfilename, 255, "%s/.%src", home, appname);
+    snprintf(appname, 255, "%s", basename(argv[0]));
+    snprintf(rcfilename, 255, "%s/.%src", home, appname);
+    
+    QApplication myApp(argc, argv);
+    
+    QTGUI* interface = new QTGUI();
+    FUI* finterface	= new FUI();
+    DSP.buildUserInterface(interface);
+    DSP.buildUserInterface(finterface);
 
-	GUI* interface = new QTGUI(argc, argv);
-	FUI* finterface	= new FUI();
-	DSP.buildUserInterface(interface);
-	DSP.buildUserInterface(finterface);
-
-#ifdef OSCCTRL
-	GUI* oscinterface = new OSCUI(appname, argc, argv);
-	DSP.buildUserInterface(oscinterface);
+#ifdef HTTPCTRL
+    httpdUI* httpdinterface = new httpdUI(appname, DSP.getNumInputs(), DSP.getNumOutputs(), argc, argv);
+    DSP.buildUserInterface(httpdinterface);
 #endif
 
-	netjackaudio audio(celt, master_ip, master_port);
-	if (!audio.init(appname, &DSP)) {
+#ifdef OSCCTRL
+    GUI* oscinterface = new OSCUI(appname, argc, argv);
+    DSP.buildUserInterface(oscinterface);
+#endif
+
+    netjackaudio audio(celt, master_ip, master_port, mtu, latency);
+    if (!audio.init(appname, &DSP)) {
         return 0;
     }
-	finterface->recallState(rcfilename);
-	if (!audio.start()) {
+    finterface->recallState(rcfilename);
+    if (!audio.start()) {
         return 0;
     }
 
-#ifdef OSCCTRL
-	oscinterface->run();
+#ifdef HTTPCTRL
+    httpdinterface->run();
 #endif
-	interface->run();
 
-	audio.stop();
-	finterface->saveState(rcfilename);
-  	return 0;
+#ifdef OSCCTRL
+    oscinterface->run();
+#endif
+    interface->run();
+	
+    myApp.setStyleSheet(interface->styleSheet());
+    myApp.exec();
+    interface->stop();
+    
+    audio.stop();
+    finterface->saveState(rcfilename);
+    
+   // desallocation
+    delete interface;
+    delete finterface;
+#ifdef HTTPCTRL
+	 delete httpdinterface;
+#endif
+#ifdef OSCCTRL
+	 delete oscinterface;
+#endif
+
+    return 0;
 }
 
-
 /********************END ARCHITECTURE SECTION (part 2/2)****************/
 
