@@ -1,7 +1,7 @@
 /************************************************************************
  ************************************************************************
     FAUST compiler
-	Copyright (C) 2003-2004 GRAME, Centre National de Creation Musicale
+    Copyright (C) 2003-2004 GRAME, Centre National de Creation Musicale
     ---------------------------------------------------------------------
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,8 +19,6 @@
  ************************************************************************
  ************************************************************************/
  
- 
- 
 /*****************************************************************************
 ******************************************************************************
 	
@@ -29,7 +27,6 @@
 ******************************************************************************
 *****************************************************************************/
 
-
 /**\file boxtype.cpp
  * \author Yann Orlarey
  * \version 1.0
@@ -38,20 +35,17 @@
  *  The type of a block diagram is defined by a number of inputs and outputs.
  */
 
-
 #include <stdio.h>
 #include <string.h>
 #include "boxes.hh"
 #include "ppbox.hh"
 #include "prim2.hh"
 #include "xtended.hh"
+#include "exception.hh"
+#include "global.hh"
 
-
-Tree BOXTYPEPROP = tree(symbol("boxTypeProp"));
 static bool infereBoxType (Tree box, int* inum, int* onum);
 
-
-
 /**
  * Return the type (number of inputs and outputs) of a box or false if undefined
  * \param box the box we want to know the type
@@ -63,7 +57,7 @@ static bool infereBoxType (Tree box, int* inum, int* onum);
 bool getBoxType (Tree box, int* inum, int* onum)
 {
 	Tree t;
-	if (getProperty(box, BOXTYPEPROP, t)) {
+	if (getProperty(box, gGlobal->BOXTYPEPROP, t)) {
 		
 		if (isNil(t)) {
 			return false;
@@ -76,17 +70,15 @@ bool getBoxType (Tree box, int* inum, int* onum)
 	} else {
 	
 		if (infereBoxType(box, inum, onum)) {
-			setProperty(box, BOXTYPEPROP, cons(tree(*inum), tree(*onum)));
+			setProperty(box, gGlobal->BOXTYPEPROP, cons(tree(*inum), tree(*onum)));
 			return true;
 		} else {
-			setProperty(box, BOXTYPEPROP, nil);
+			setProperty(box, gGlobal->BOXTYPEPROP, gGlobal->nil);
 			return false;
 		}
 	}
 }
 
-
-
 /**
  * Infere the type (number of inputs and outputs) of a box.
  * The box expression is assumed to be in 'propagation normal form'
@@ -100,7 +92,7 @@ bool getBoxType (Tree box, int* inum, int* onum)
 
 static bool infereBoxType (Tree t, int* inum, int* onum)
 {
-	Tree a, b, ff, l, s;
+	Tree a, b, ff, l, s, c;
 	//Tree abstr, genv, vis, lenv;
 	
 	xtended* p = (xtended*) getUserData(t);
@@ -108,6 +100,9 @@ static bool infereBoxType (Tree t, int* inum, int* onum)
 	if (p) 						{ *inum = p->arity(); *onum = 1; }
 	else if (isBoxInt(t)) 		{ *inum = 0; *onum = 1; } 
 	else if (isBoxReal(t)) 		{ *inum = 0; *onum = 1; } 
+	
+	else if (isBoxWaveform(t)) 	{ *inum = 0; *onum = 2; } 
+
 	else if (isBoxWire(t)) 		{ *inum = 1; *onum = 1; }
 	else if (isBoxCut(t)) 		{ *inum = 1; *onum = 0; } 
 
@@ -137,21 +132,25 @@ static bool infereBoxType (Tree t, int* inum, int* onum)
     else if (isBoxTGroup(t,l,a)){ return getBoxType(a, inum, onum); }
 	
 	else if (isBoxVBargraph(t)) 	{ *inum = 1; *onum = 1; } 
-	else if (isBoxHBargraph(t)) 	{ *inum = 1; *onum = 1; } 
-
+	else if (isBoxHBargraph(t)) 	{ *inum = 1; *onum = 1; }
+    else if (isBoxSoundfile(t, l, c)) {
+        *inum = 1;
+        *onum = 2+tree2int(c);
+    }
 	else if (isBoxSeq(t, a, b)) {
-		
 		int u,v,x,y;
 		if (!getBoxType(a, &u, &v)) return false;
 		if (!getBoxType(b, &x, &y)) return false;
 
 		if (v != x) {
-            cerr    << "Error in sequential composition (A:B)" << endl
+            stringstream error;
+            error   << "Error in sequential composition (A:B)" << endl
                     << "The number of outputs (" << v << ") of A = " << boxpp(a) << endl
                     << "must be equal to the number of inputs (" << x << ") of B : " << boxpp(b) << endl;
-            exit(1);
+            throw faustexception(error.str());
 		} else {
-			*inum = u; *onum = y;
+            *inum = u;
+            *onum = y;
 		}
 
 	} else if (isBoxPar(t, a, b)) {
@@ -169,23 +168,26 @@ static bool infereBoxType (Tree t, int* inum, int* onum)
 		if (!getBoxType(b, &x, &y)) return false;
 
         if (v == 0) {
-            cerr    << "Connection error in : " << boxpp(t) << endl
+            stringstream error;
+            error   << "Connection error in : " << boxpp(t) << endl
                     << "The first expression : " << boxpp(a) << " has no outputs" << endl;
-            exit(1);
+            throw faustexception(error.str());
         }
         
         if (x == 0) {
-            cerr    << "Connection error in : " << boxpp(t) << endl
+            stringstream error;
+            error   << "Connection error in : " << boxpp(t) << endl
                     << "The second expression : " << boxpp(b) << " has no inputs" << endl;
-            exit(1);
+            throw faustexception(error.str());
         }
 		 
 		if (x % v != 0) {
-			cerr 	<< "Connection error in : " << boxpp(t) << endl
+            stringstream error;
+            error   << "Connection error in : " << boxpp(t) << endl
 					<< "The number of outputs " << v
 					<< " of the first expression should be a divisor of the number of inputs " << x
 					<< " of the second expression" << endl;
-			exit(1);
+			throw faustexception(error.str());
 		}
 		
 		*inum = u; *onum = y;
@@ -197,23 +199,26 @@ static bool infereBoxType (Tree t, int* inum, int* onum)
 		if (!getBoxType(b, &x, &y)) return false;
 
         if (v == 0) {
-            cerr    << "Connection error in : " << boxpp(t) << endl
+            stringstream error;
+            error   << "Connection error in : " << boxpp(t) << endl
                     << "The first expression : " << boxpp(a) << " has no outputs" << endl;
-            exit(1);
+            throw faustexception(error.str());
         }
         
         if (x == 0) {
-            cerr    << "Connection error in : " << boxpp(t) << endl
+            stringstream error;
+            error   << "Connection error in : " << boxpp(t) << endl
                     << "The second expression : " << boxpp(b) << " has no inputs" << endl;
-            exit(1);
+            throw faustexception(error.str());
         }
         
 		if (v % x != 0) { 
-			cerr 	<< "Connection error in : " << boxpp(t) << endl
+			stringstream error;
+            error   << "Connection error in : " << boxpp(t) << endl
 					<< "The number of outputs " << v
 					<< " of the first expression should be a multiple of the number of inputs " << x
 					<< " of the second expression" << endl;
-			exit(1);
+            throw faustexception(error.str());
 		}
 
 		*inum = u; *onum = y;
@@ -224,23 +229,26 @@ static bool infereBoxType (Tree t, int* inum, int* onum)
 		if (!getBoxType(a, &u, &v)) return false;
 		if (!getBoxType(b, &x, &y)) return false;
 		if ( (x > v) | (y > u) ) { 
-			cerr 	<< "Connection error in : " << boxpp(t) << endl;
-			if (x > v) cerr << "The number of outputs " << v 
-							<< " of the first expression should be greater or equal \n  to the number of inputs " << x 
+			stringstream error;
+            error	<< "Connection error in : " << boxpp(t) << endl;
+			if (x > v) error << "The number of outputs " << v 
+                            << " of the first expression should be greater or equal \n  to the number of inputs " << x 
 							<< " of the second expression" << endl;
-			if (y > u) cerr	<< "The number of inputs " << u
+			if (y > u) error << "The number of inputs " << u
 							<< " of the first expression should be greater or equal \n  to the number of outputs " << y
 							<< " of the second expression" << endl;
-			exit(1);
+			throw faustexception(error.str());
 		}
 		*inum = max(0,u-y); *onum = v;
 		
     } else if (isBoxEnvironment(t)) {
-        cerr << "Connection error : an environment is not a block-diagram : " << boxpp(t) << endl;
-        exit(1);
+        stringstream error;
+        error << "Connection error : an environment is not a block-diagram : " << boxpp(t) << endl;
+        throw faustexception(error.str());
     } else {
-        cerr << "boxType() internal error : unrecognized box expression " << boxpp(t) << endl;
-        exit(1);
+        stringstream error;
+        error << "boxType() internal error : unrecognized box expression " << boxpp(t) << endl;
+        throw faustexception(error.str());
 	}
 	return true;
 }	
