@@ -26,7 +26,6 @@
  * ps format. Complex block-diagrams are automatically splitted.
  */
 
-
 #include <stdio.h>
 #include <ctype.h>
 #include <sys/stat.h>
@@ -45,13 +44,14 @@
 #include "boxes.hh"
 #include "ppbox.hh"
 #include "prim2.hh"
+#include "global.hh"
 
 #include <vector>
 #include "devLib.h"
 #include "ppbox.hh"
 #include "xtended.hh"
 #include "occurrences.hh"
-#include "boxcomplexity.h"
+#include "boxcomplexity.hh"
 
 #include "schema.h"
 #include "drawschema.hh"
@@ -59,8 +59,8 @@
 #include "names.hh"
 #include "description.hh"
 #include "property.hh"
-
-
+#include "exception.hh"
+#include "files.hh"
 
 #if 0
 #define linkcolor "#b3d1dc"
@@ -122,20 +122,6 @@
 
 using namespace std;
 
-// external parameters
-extern int gFoldThreshold;				// max diagram complexity before folding
-
-
-// internal state during drawing
-static Occurrences* 	gOccurrences;
-static bool				sFoldingFlag;		// true with complex block-diagrams
-static stack<Tree>		gPendingExp;		// Expressions that need to be drawn
-static set<Tree>		gDrawnExp;			// Expressions drawn or scheduled so far
-static const char* 		gDevSuffix;			// .svg or .ps used to choose output device
-static char 			gCurrentDir[512];	// room to save current directory name
-static string			gSchemaFileName;	// name of schema file beeing generated
-static map<Tree,string>	gBackLink;			// link to enclosing file for sub schema
-
 // prototypes of internal functions
 static void 	writeSchemaFile(Tree bd);
 static schema* 	generateDiagramSchema (Tree bd);
@@ -147,12 +133,11 @@ static schema* 	generateOutputSlotSchema(Tree a);
 static schema* 	generateInputSlotSchema(Tree a);
 static schema* 	generateBargraphSchema(Tree t);
 static schema* 	generateUserInterfaceSchema(Tree t);
+static schema* 	generateSoundfileSchema(Tree t);
 static char* 	legalFileName(Tree t, int n, char* dst);
-static int 		cholddir ();
-static int 		mkchdir(const char* dirname);
-
-
 
+static schema*  addSchemaInputs(int ins, schema* x);
+static schema*  addSchemaOutputs(int outs, schema* x);
 
 /**
  *The entry point to generate from a block diagram as a set of
@@ -161,9 +146,9 @@ static int 		mkchdir(const char* dirname);
  */
 void drawSchema(Tree bd, const char* projname, const char* dev)
 {
-	gDevSuffix 		= dev;
-	sFoldingFlag 	= boxComplexity(bd) > gFoldThreshold;
-
+	gGlobal->gDevSuffix 	= dev;
+	gGlobal->gFoldingFlag 	= boxComplexity(bd) > gGlobal->gFoldThreshold;
+   
 	mkchdir(projname); 			// create a directory to store files
 
 	scheduleDrawing(bd);		// schedule the initial drawing
@@ -190,10 +175,10 @@ void drawSchema(Tree bd, const char* projname, const char* dev)
  */
 static void scheduleDrawing(Tree t)
 {
-	if (gDrawnExp.find(t) == gDrawnExp.end()) {
-		gDrawnExp.insert(t);
-		gBackLink.insert(make_pair(t,gSchemaFileName));	// remember the enclosing filename
-		gPendingExp.push(t);
+	if (gGlobal->gDrawnExp.find(t) == gGlobal->gDrawnExp.end()) {
+		gGlobal->gDrawnExp.insert(t);
+		gGlobal->gBackLink.insert(make_pair(t,gGlobal->gSchemaFileName));	// remember the enclosing filename
+		gGlobal->gPendingExp.push(t);
 	}
 }
 
@@ -202,14 +187,12 @@ static void scheduleDrawing(Tree t)
  */
 static bool pendingDrawing(Tree& t)
 {
-	if (gPendingExp.empty()) return false;
-	t = gPendingExp.top();
-	gPendingExp.pop();
+	if (gGlobal->gPendingExp.empty()) return false;
+	t = gGlobal->gPendingExp.top();
+	gGlobal->gPendingExp.pop();
 	return true;
 }
 
-
-
 //------------------------ dealing with files -------------------------
 
 /**
@@ -221,82 +204,52 @@ static void writeSchemaFile(Tree bd)
 {
 	Tree			id;
 	schema* 		ts;
+    int             ins, outs;
 
 	char 			temp[1024];
 
-	gOccurrences = new Occurrences(bd);
+	gGlobal->gOccurrences = new Occurrences(bd);
+    getBoxType(bd, &ins, &outs);
 
 	bool hasname = getDefNameProperty(bd, id); 
 
-	//assert(hasname);
+	//faustassert(hasname);
 	if (!hasname) {
 		// create an arbitrary name 
 		id = tree(Node(unique("diagram_")));
 	}
 
 	// generate legal file name for the schema
-	stringstream s1; s1 << legalFileName(bd, 1024, temp) << "." << gDevSuffix;
-	gSchemaFileName = s1.str();
+    stringstream s1; s1 << legalFileName(bd, 1024, temp) << "." << gGlobal->gDevSuffix;
+    string res1 = s1.str();
+    gGlobal->gSchemaFileName = res1;
 
 	// generate the label of the schema
 	stringstream s2; s2 << tree2str(id);
-	string link = gBackLink[bd];
-	ts = makeTopSchema(generateInsideSchema(bd), 20, s2.str(), link);
+	string link = gGlobal->gBackLink[bd];
+    ts = makeTopSchema(addSchemaOutputs(outs, addSchemaInputs(ins, generateInsideSchema(bd))), 20, s2.str(), link);
 	// draw to the device defined by gDevSuffix
-	if (strcmp(gDevSuffix, "svg") == 0) {
-		SVGDev dev(s1.str().c_str(), ts->width(), ts->height());
-		ts->place(0,0, kLeftRight);
-		ts->draw(dev);
-        { collector c; ts->collectTraits(c); c.draw(dev); }
-	} else {
-		PSDev dev(s1.str().c_str(), ts->width(), ts->height());
-		ts->place(0,0, kLeftRight);
-		ts->draw(dev);
+    if (strcmp(gGlobal->gDevSuffix, "svg") == 0) {
+        SVGDev dev(res1.c_str(), ts->width(), ts->height());
+        ts->place(0,0, kLeftRight);
+        ts->draw(dev);
         {
             collector c;
             ts->collectTraits(c);
             c.draw(dev);
         }
-	}
+    } else {
+        PSDev dev(res1.c_str(), ts->width(), ts->height());
+        ts->place(0,0, kLeftRight);
+        ts->draw(dev);
+        {
+            collector c;
+            ts->collectTraits(c);
+            c.draw(dev);
+        }
+    }
 }
 
-
-
-/**
- * Create a new directory in the current one to store the diagrams.
- * The current directory is saved to be later restaured.
- */
-static int mkchdir(const char* dirname)
-{
-	//cerr << "mkchdir of " << dirname << endl;
-	if (getcwd(gCurrentDir, 512) != 0) {
-		int status = mkdir(dirname, S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);
-		if (status == 0 || errno == EEXIST) {
-			if (chdir(dirname) == 0) {
-				return 0;
-			}
-		}
-	}
-	perror("mkchdir");
-	exit(errno);
-	//return errno;
-}
-
-
-/**
- *Switch back to the previously stored current directory
- */
-static int cholddir ()
-{
-	if (chdir(gCurrentDir) == 0) {
-		return 0;
-	} else {
-		perror("cholddir");
-		exit(errno);
-	}
-}
-
-
 /**
  * Transform the definition name property of tree <t> into a
  * legal file name.  The resulting file name is stored in
@@ -316,13 +269,12 @@ static char* legalFileName(Tree t, int n, char* dst)
 	dst[i] = 0;
 	if (strcmp(dst, "process") != 0) { 
 		// if it is not process add the hex address to make the name unique
-		snprintf(&dst[i], n-i, "-%p", t);
+		snprintf(&dst[i], n-i, "-%p", (void*)t);
 	}
 	return dst;
 }
 
 
-
 //------------------------ generating the schema -------------------------
 
 
@@ -357,7 +309,6 @@ static bool isInverter(Tree t)
  * only made of cut, wires and slots. No labels will be
  * dispayed for pure routing expressions.
  */
-property<bool> gPureRoutingProperty;
 
 static bool isPureRouting(Tree t)
 {
@@ -365,7 +316,7 @@ static bool isPureRouting(Tree t)
     int     ID;
     Tree    x,y;
 
-    if (gPureRoutingProperty.get(t,r)) {
+    if (gGlobal->gPureRoutingProperty->get(t,r)) {
         return r;
     } else if (    isBoxCut(t)
                 || isBoxWire(t)
@@ -376,10 +327,10 @@ static bool isPureRouting(Tree t)
                 || (isBoxSplit(t,x,y) && isPureRouting(x) && isPureRouting(y))
                 || (isBoxMerge(t,x,y) && isPureRouting(x) && isPureRouting(y))
               ) {
-        gPureRoutingProperty.set(t,true);
+        gGlobal->gPureRoutingProperty->set(t,true);
         return true;
     } else {
-        gPureRoutingProperty.set(t,false);
+        gGlobal->gPureRoutingProperty->set(t,false);
         return false;
     }
 }
@@ -403,13 +354,13 @@ static schema* generateDiagramSchema(Tree t)
 		//cerr << t << "\tNAMED : " << s.str() << endl;
 	}
 
-	if ( sFoldingFlag && /*(gOccurrences->getCount(t) > 0) &&*/
+	if (gGlobal->gFoldingFlag && /*(gOccurrences->getCount(t) > 0) &&*/
 			(boxComplexity(t) > 2) && getDefNameProperty(t, id)) {
 		char 	temp[1024];
 		getBoxType(t, &ins, &outs);
 		stringstream s, l;
 		s << tree2str(id);
-		l << legalFileName(t,1024,temp) << "." << gDevSuffix;
+		l << legalFileName(t,1024,temp) << "." << gGlobal->gDevSuffix;
 		scheduleDrawing(t);
 		return makeBlockSchema(ins, outs, s.str(), linkcolor, l.str());
 
@@ -452,7 +403,8 @@ static schema* generateInsideSchema(Tree t)
 
 	else if (isBoxInt(t, &i))		{ stringstream 	s; s << i; return makeBlockSchema(0, 1, s.str(), numcolor, "" ); }
 	else if (isBoxReal(t, &r)) 		{ stringstream 	s; s << r; return makeBlockSchema(0, 1, s.str(), numcolor, "" ); }
-	else if (isBoxWire(t)) 			{ return makeCableSchema(); }
+	else if (isBoxWaveform(t))      { return makeBlockSchema(0, 2, "waveform{...}", normalcolor, ""); }
+    else if (isBoxWire(t)) 			{ return makeCableSchema(); }
 	else if (isBoxCut(t)) 			{ return makeCutSchema();  }
 
 	else if (isBoxPrim0(t, &p0)) 	{ return makeBlockSchema(0, 1, prim0name(p0), normalcolor, ""); }
@@ -473,6 +425,7 @@ static schema* generateInsideSchema(Tree t)
 	else if (isBoxNumEntry(t)) 		{ return generateUserInterfaceSchema(t); }
 	else if (isBoxVBargraph(t))		{ return generateBargraphSchema(t); }
 	else if (isBoxHBargraph(t))		{ return generateBargraphSchema(t); }
+	else if (isBoxSoundfile(t))		{ return generateSoundfileSchema(t); }
 
 	// don't draw group rectangle when labels are empty (ie "")
     else if (isBoxVGroup(t,l,a))	{ 	stringstream s; s << "vgroup(" << extractName(l) << ")";
@@ -499,13 +452,12 @@ static schema* generateInsideSchema(Tree t)
 		} else {
 			return makeDecorateSchema(generateAbstractionSchema(generateInputSlotSchema(a), b), 10, "Abstraction");
 		}
-	}
-
-	else {
-
-		fprintf(stderr, "Internal Error, box expression not recognized : "); print(t, stderr); fprintf(stderr, "\n");
-		exit(1);
-
+	} else {
+        stringstream error;
+        error << "Internal Error, box expression not recognized : ";
+        t->print(error);
+        error << endl;
+        throw faustexception(error.str());
 	}
 }
 
@@ -514,7 +466,7 @@ static schema* generateInsideSchema(Tree t)
  */
 static void UserInterfaceDescription(Tree box, string& d)
 {
-    Tree    t1, label, cur, min, max, step;
+    Tree    t1, label, cur, min, max, step, chan;
     stringstream 	fout;
     // user interface
          if (isBoxButton(box, label))	fout << "button(" << extractName(label) << ')';
@@ -564,9 +516,13 @@ static void UserInterfaceDescription(Tree box, string& d)
              << boxpp(max) << ", "
              << boxpp(step)<< ')';
     }
+    else if (isBoxSoundfile(box, label, chan)) 	{
+        fout << "soundfile("
+             << extractName(label) << ", "
+             << boxpp(chan) << ')';
+    }
     else {
-        cerr << "INTERNAL ERROR : unknow user interface element " << endl;
-        exit(0);
+        throw faustexception("INTERNAL ERROR : unknown user interface element\n");
     }
     d = fout.str();
 }
@@ -591,6 +547,21 @@ static schema* generateBargraphSchema(Tree t)
     return makeBlockSchema(1, 1, s, uicolor, "");
 }
 
+/**
+ * Generate a 1->c+2 block schema for soundfile("toto",c) 
+ */
+static schema* generateSoundfileSchema(Tree t)
+{
+    Tree    label, chan;
+    if (isBoxSoundfile(t, label, chan)) {
+        int n = tree2int(chan);
+        string s; UserInterfaceDescription(t,s);
+        return makeBlockSchema(1, 2+n, s, uicolor, "");
+    } else {
+        throw faustexception("Internal error\n");
+    }
+}
+
 
 
 /**
@@ -598,7 +569,7 @@ static schema* generateBargraphSchema(Tree t)
  */
 static schema* generateInputSlotSchema(Tree a)
 {
-	Tree id; assert(getDefNameProperty(a, id));
+	Tree id; faustassert(getDefNameProperty(a, id));
 	stringstream s; s << tree2str(id);
 	return makeBlockSchema(1, 0, s.str(), slotcolor, "");
 }
@@ -610,7 +581,7 @@ static schema* generateInputSlotSchema(Tree a)
  */
 static schema* generateOutputSlotSchema(Tree a)
 {
-	Tree id; assert(getDefNameProperty(a, id));
+	Tree id; faustassert(getDefNameProperty(a, id));
 	stringstream s; s << tree2str(id);
 	return makeBlockSchema(0, 1, s.str(), slotcolor, "");
 }
@@ -632,3 +603,39 @@ static schema* generateAbstractionSchema(schema* x, Tree t)
 	return makeSeqSchema(x, generateDiagramSchema(t));
 }
 
+static schema* addSchemaInputs(int ins, schema* x)
+{
+    if (ins==0) {
+        return x;
+    } else {
+        schema* y = 0;
+        do {
+            schema* z = makeConnectorSchema();
+            if (y != 0) {
+                y = makeParSchema(y,z);
+            } else {
+                y = z;
+            }
+        } while (--ins);
+        return makeSeqSchema(y,x);
+    }
+}
+
+
+static schema* addSchemaOutputs(int outs, schema* x)
+{
+    if (outs==0) {
+        return x;
+    } else {
+        schema* y = 0;
+        do {
+            schema* z = makeConnectorSchema();
+            if (y != 0) {
+                y = makeParSchema(y,z);
+            } else {
+                y = z;
+            }
+        } while (--outs);
+        return makeSeqSchema(x,y);
+    }
+}
