@@ -19,9 +19,10 @@
  ************************************************************************
  ************************************************************************/
 
+#include <algorithm>
 
 #include "blockSchema.h"
-#include <assert.h>
+#include "exception.hh"
 
 using namespace std;
 
@@ -31,24 +32,25 @@ static double quantize(int n)
 	return dLetter * (q *((n+q-1)/q));
 }
 
+
 /**
  * Build a simple colored blockSchema with a certain number of
  * inputs and outputs, a text to be displayed, and an optional link.
  * Computes the size of the box according to the length of the text
  * and the maximum number of ports.
  */
-schema* makeBlockSchema (	unsigned int inputs,
-							unsigned int outputs,
-							const string& text,
-							const string& color,
-							const string& link )
+schema* makeBlockSchema (unsigned int inputs,
+                        unsigned int outputs,
+                        const string& text,
+                        const string& color,
+                        const string& link)
 {
-	// determine the optimal size of the box
-	double minimal = 3*dWire;
-	double w = 2*dHorz + max( minimal, quantize(text.size()) );
-	double h = 2*dVert + max( minimal, max(inputs, outputs) * dWire );
+    // determine the optimal size of the box
+    double minimal = 3*dWire;
+    double w = 2*dHorz + max( minimal, quantize((int)text.size()) );
+    double h = 2*dVert + max( minimal, max(inputs, outputs) * dWire );
 
-	return new blockSchema(inputs, outputs, w, h, text, color, link);
+    return new blockSchema(inputs, outputs, w, h, text, color, link);
 }
 
 /**
@@ -57,18 +59,18 @@ schema* makeBlockSchema (	unsigned int inputs,
  * The length of the text as well as th number of inputs and outputs
  * are used to compute the size of the blockSchema
  */
-blockSchema::blockSchema (	unsigned int inputs,
-							unsigned int outputs,
-							double width,
-							double height,
-							const string& text,
-							const string& color,
-							const string& link )
+blockSchema::blockSchema (unsigned int inputs,
+                        unsigned int outputs,
+                        double width,
+                        double height,
+                        const string& text,
+                        const string& color,
+                        const string& link)
 
 	: 	schema( inputs, outputs, width, height ),
 	  	fText(text),
 	  	fColor(color),
-	  	fLink(link)
+        fLink(link)
 {
     for (unsigned int i=0; i<inputs; i++) 	fInputPoint.push_back(point(0,0));
     for (unsigned int i=0; i<outputs; i++) 	fOutputPoint.push_back(point(0,0));
@@ -94,8 +96,8 @@ void blockSchema::place(double x, double y, int orientation)
  */
 point blockSchema::inputPoint(unsigned int i) const
 {
-	assert (placed());
-	assert (i < inputs());
+	faustassert(placed());
+	faustassert(i < inputs());
 	return fInputPoint[i];
 }
 
@@ -104,8 +106,8 @@ point blockSchema::inputPoint(unsigned int i) const
  */
 point blockSchema::outputPoint(unsigned int i) const
 {
-	assert (placed());
-	assert (i < outputs());
+	faustassert(placed());
+	faustassert(i < outputs());
 	return fOutputPoint[i];
 }
 
@@ -115,7 +117,7 @@ point blockSchema::outputPoint(unsigned int i) const
  */
 void blockSchema::placeInputPoints()
 {
-	int		N = inputs();
+	int N = inputs();
 
 	if (orientation() == kLeftRight) {
 
@@ -137,7 +139,6 @@ void blockSchema::placeInputPoints()
 	}
 }
 
-
 /**
  * Computes the output points according to the position and the
  * orientation of the blockSchema
@@ -166,20 +167,18 @@ void blockSchema::placeOutputPoints()
 	}
 }
 
-
 /**
  * Draw the blockSchema on the device. This methos can only
  * be called after the blockSchema have been placed
  */
 void blockSchema::draw(device& dev)
 {
-	assert(placed());
+	faustassert(placed());
 
 	drawRectangle(dev);
 	drawText(dev);
-	drawOrientationMark(dev);
-    drawInputWires(dev);
-    drawOutputWires(dev);
+    drawOrientationMark(dev);
+    drawInputArrows(dev);
 }
 
 /**
@@ -196,7 +195,6 @@ void blockSchema::drawRectangle(device& dev)
 			);
 }
 
-
 /**
  * Draw the text centered on the box
  */
@@ -209,7 +207,6 @@ void blockSchema::drawText(device& dev)
 			);
 }
 
-
 /**
  * Draw the orientation mark, a small point that indicates
  * the first input (like integrated circuits)
@@ -228,37 +225,21 @@ void blockSchema::drawOrientationMark(device& dev)
 
 	dev.markSens( px, py, orientation() );
 }
-#if 1
+
 /**
  * Draw horizontal arrows from the input points to the
  * blockSchema rectangle
  */
-void blockSchema::drawInputWires(device& dev)
+void blockSchema::drawInputArrows(device& dev)
 {
     double dx = (orientation() == kLeftRight) ? dHorz : -dHorz;
 
     for (unsigned int i=0; i<inputs(); i++) {
         point p = fInputPoint[i];
-        //dev.trait(p.x, p.y, p.x+dx, p.y);
         dev.fleche(p.x+dx, p.y, 0, orientation());
     }
 }
 
-/**
- * Draw horizontal line from the blockSchema rectangle to the
- * output points
- */
-void blockSchema::drawOutputWires(device& dev)
-{
-    //double dx = (orientation() == kLeftRight) ? dHorz : -dHorz;
-
-    for (unsigned int i=0; i<outputs(); i++) {
-        point p = fOutputPoint[i];
-        //dev.trait(p.x, p.y, p.x-dx, p.y);
-    }
-}
-#endif
-
 /**
  * Draw horizontal arrows from the input points to the
  * blockSchema rectangle
