@@ -19,10 +19,11 @@
  ************************************************************************
  ************************************************************************/
 
-#include "enlargedSchema.h"
-#include <assert.h>
 #include <iostream>
 
+#include "enlargedSchema.h"
+#include "exception.hh"
+
 using namespace std;
 
 /**
@@ -46,11 +47,10 @@ enlargedSchema::enlargedSchema( schema* s, double width )
 	: 	schema(s->inputs(), s->outputs(), width, s->height()),
 	 	fSchema(s)
 {
-	for (unsigned int i=0; i<inputs(); i++) 	fInputPoint.push_back(point(0));
-	for (unsigned int i=0; i<outputs(); i++) 	fOutputPoint.push_back(point(0));
+    for (unsigned int i=0; i<inputs(); i++) 	fInputPoint.push_back(point(0,0));
+    for (unsigned int i=0; i<outputs(); i++) 	fOutputPoint.push_back(point(0,0));
 }
 
-
 /**
  * Define the graphic position of the schema. Computes the graphic
  * position of all the elements, in particular the inputs and outputs.
@@ -69,12 +69,12 @@ void enlargedSchema::place(double ox, double oy, int orientation)
 
 	for (unsigned int i=0; i < inputs(); i++) {
 		point p = fSchema->inputPoint(i);
-		fInputPoint[i] = point(p.x-dx, p.y);
+        fInputPoint[i] = point(p.x-dx, p.y); //, p.z);
 	}
 
 	for (unsigned int i=0; i < outputs(); i++) {
 		point p = fSchema->outputPoint(i);
-		fOutputPoint[i] = point(p.x+dx, p.y);
+        fOutputPoint[i] = point(p.x+dx, p.y); //, p.z);
 	}
 
 	endPlace();
@@ -85,8 +85,8 @@ void enlargedSchema::place(double ox, double oy, int orientation)
  */
 point enlargedSchema::inputPoint(unsigned int i)	const
 {
-	assert (placed());
-	assert (i < inputs());
+	faustassert(placed());
+	faustassert(i < inputs());
 	return fInputPoint[i];
 }
 
@@ -95,8 +95,8 @@ point enlargedSchema::inputPoint(unsigned int i)	const
  */
 point enlargedSchema::outputPoint(unsigned int i)	const
 {
-	assert (placed());
-	assert (i < outputs());
+	faustassert(placed());
+	faustassert(i < outputs());
 	return fOutputPoint[i];
 }
 
@@ -106,21 +106,47 @@ point enlargedSchema::outputPoint(unsigned int i)	const
  */
 void enlargedSchema::draw(device& dev)
 {
-	assert(placed());
+    faustassert(placed());
 
-	fSchema->draw(dev);
+    fSchema->draw(dev);
+#if 0
+    // draw enlarge input wires
+    for (unsigned int i=0; i<inputs(); i++) {
+        point p = inputPoint(i);
+        point q = fSchema->inputPoint(i);
+        if ( (p.z>=0) && (q.z>=0) ) dev.trait(p.x, p.y, q.x, q.y);
+    }
 
-	// draw enlarge input wires
-	for (unsigned int i=0; i<inputs(); i++) {
-		point p = inputPoint(i);
-		point q = fSchema->inputPoint(i);
-		dev.trait(p.x, p.y, q.x, q.y);
-	}
-
-	// draw enlarge output wires
-	for (unsigned int i=0; i<outputs(); i++) {
-		point p = outputPoint(i);
-		point q = fSchema->outputPoint(i);
-		dev.trait(p.x, p.y, q.x, q.y);
-	}
+    // draw enlarge output wires
+    for (unsigned int i=0; i<outputs(); i++) {
+        point p = outputPoint(i);
+        point q = fSchema->outputPoint(i);
+        if ( (p.z>=0) && (q.z>=0) ) dev.trait(p.x, p.y, q.x, q.y);
+    }
+#endif
+}
+
+/**
+ * Draw the enlarged schema. This methos can only
+ * be called after the block have been placed
+ */
+void enlargedSchema::collectTraits(collector& c)
+{
+    faustassert(placed());
+
+    fSchema->collectTraits(c);
+
+    // draw enlarge input wires
+    for (unsigned int i=0; i<inputs(); i++) {
+        point p = inputPoint(i);
+        point q = fSchema->inputPoint(i);
+        c.addTrait(trait(p,q));     // in->out direction
+    }
+
+    // draw enlarge output wires
+    for (unsigned int i=0; i<outputs(); i++) {
+        point q = fSchema->outputPoint(i);
+        point p = outputPoint(i);
+        c.addTrait(trait(q,p));     // in->out direction
+    }
 }
