@@ -19,10 +19,11 @@
  ************************************************************************
  ************************************************************************/
 
-
-#include "recSchema.h"
 #include <iostream>
-#include <assert.h>
+#include <algorithm>
+
+#include "exception.hh"
+#include "recSchema.h"
 
 using namespace std;
 
@@ -54,9 +55,9 @@ recSchema::recSchema (schema* s1, schema* s2, double width)
 		fSchema2(s2)
 {
 	// this version only accepts legal expressions of same width
-	assert(s1->inputs() >= s2->outputs());
-	assert(s1->outputs() >= s2->inputs());
-	assert(s1->width() >= s2->width());
+	faustassert(s1->inputs() >= s2->outputs());
+	faustassert(s1->outputs() >= s2->inputs());
+	faustassert(s1->width() >= s2->width());
 
 	// create the input and output points
     for (unsigned int i=0; i<inputs(); i++) 	fInputPoint.push_back(point(0,0));
@@ -123,45 +124,26 @@ point recSchema::outputPoint(unsigned int i) const
 
 
 /**
- * Draw the two subschema s1 and s2 as well as the feedback
- * connections between s1 and s2, and the feedfrom connections
- * beetween s2 and s1.
+ * Draw the two subschema s1 and s2 as well as the implicit feedback
+ * delays between s1 and s2.
  */
 void recSchema::draw(device& dev)
 {
-    assert(placed());
+    faustassert(placed());
 
     // draw the two subdiagrams
     fSchema1->draw(dev);
     fSchema2->draw(dev);
 
-    // draw the output lines
-    for (unsigned int i=0; i<outputs(); i++) {
-        point p = fSchema1->outputPoint(i);
-        point q = outputPoint(i);
-        //dev.trait(p.x, p.y, q.x, q.y);
-    }
-
-    // draw the input lines
-    unsigned int skip = fSchema2->outputs();
-    for (unsigned int i=0; i<inputs(); i++) {
-        point p = fSchema1->inputPoint(i+skip);
-        point q = inputPoint(i);
-        //dev.trait(p.x, p.y, q.x, q.y);
-    }
-
-    // draw the feedback connections to each fSchema2 input
+    // draw the implicit feedback delay to each fSchema2 input
+    double dw = (orientation()==kLeftRight) ? dWire : -dWire;
     for (unsigned int i=0; i<fSchema2->inputs(); i++) {
-        drawFeedback(dev, fSchema1->outputPoint(i), fSchema2->inputPoint(i), i*dWire);
-    }
+        const point& p = fSchema1->outputPoint(i);
+        drawDelaySign(dev, p.x + i*dw, p.y, dw/2);
 
-    // draw the feedfront connections from each fSchema2 output
-    for (unsigned int i=0; i<fSchema2->outputs(); i++) {
-        drawFeedfront(dev, fSchema2->outputPoint(i), fSchema1->inputPoint(i), i*dWire);
     }
 }
 
-
 /**
  * Draw the delay sign of a feedback connection
  */
@@ -180,7 +162,7 @@ void recSchema::drawDelaySign(device& dev, double x, double y, double size)
  */
 void recSchema::collectTraits(collector& c)
 {
-    assert(placed());
+    faustassert(placed());
 
     // draw the two subdiagrams
     fSchema1->collectTraits(c);
@@ -214,22 +196,6 @@ void recSchema::collectTraits(collector& c)
 
 
 
-/**
- * Draw a feedback connection between two points with an horizontal
- * displacement dx
- */
-void recSchema::drawFeedback(device& dev, const point& src, const point& dst, double dx)
-{
-    double	ox = src.x + ((orientation()==kLeftRight) ? dx : -dx);
-    double	ct = (orientation()==kLeftRight) ? dWire/2 : -dWire/2;
-
-    drawDelaySign(dev, ox, src.y, ct);
-    //dev.trait(ox, src.y-ct, ox, dst.y);
-    //dev.trait(ox, dst.y, dst.x, dst.y);
-}
-
-
-
 /**
  * Draw a feedback connection between two points with an horizontal
  * displacement dx
@@ -254,20 +220,6 @@ void recSchema::collectFeedback(collector& c, const point& src, const point& dst
 }
 
 
-/**
- * Draw a feedfrom connection between two points with an horizontal
- * displacement dx
- */
-void recSchema::drawFeedfront(device& dev, const point& src, const point& dst, double dx)
-{
-//    double	ox = src.x + ((orientation()==kLeftRight) ? -dx : dx);
-
-//	dev.trait(ox, src.y, src.x, src.y);
-//	dev.trait(ox, src.y, ox, dst.y);
-//	dev.trait(ox, dst.y, dst.x, dst.y);
-}
-
-
 /**
  * Draw a feedfrom connection between two points with an horizontal
  * displacement dx
