@@ -22,8 +22,9 @@
 #ifndef __SCHEMA__
 #define __SCHEMA__
 
-
 #include "device.h"
+#include "garbageable.hh"
+
 #include <vector>
 #include <string>
 #include <set>
@@ -36,12 +37,12 @@ const double dLetter = 4.3;			///< width of a letter
 const double dHorz = 4;       		///< marge horizontale
 const double dVert = 4;				///< marge verticale
 
-
-struct point
+struct point  : public virtual Garbageable
 {
     double  x;
     double  y;
 
+    point() : x(0.0), y(0.0) {}
     point(double u, double v) : x(u), y(v) {}
     point(const point& p) : x(p.x), y(p.y) {}
 
@@ -53,7 +54,7 @@ struct point
     }
 };
 
-struct trait
+struct trait : public virtual Garbageable
 {
     point   start;
     point   end;
@@ -71,7 +72,7 @@ struct trait
     }
 };
 
-struct collector
+struct collector : public virtual Garbageable 
 {
     set<point>  fOutputs;       // collect real outputs
     set<point>  fInputs;        // collect real inputs
@@ -89,12 +90,11 @@ struct collector
 
 enum { kLeftRight=1, kRightLeft=-1 };
 
-
-
 /**
  * An abstract block diagram schema
  */
-class schema
+ 
+class schema : public virtual Garbageable
 {
   private:
 	const unsigned int	fInputs;
@@ -143,9 +143,9 @@ class schema
  	// abstract interface for subclasses
 	virtual void 	place(double x, double y, int orientation) 	= 0;
     virtual void 	draw(device& dev) 							= 0;
-    virtual void 	collectTraits(collector& c)					= 0;
     virtual point	inputPoint(unsigned int i) const			= 0;
 	virtual point 	outputPoint(unsigned int i)const			= 0;
+    virtual void 	collectTraits(collector& c)					= 0;
 };
 
 // various functions to create schemas
@@ -155,7 +155,6 @@ schema* makeBlockSchema 	(unsigned int inputs,
   							 const string& name,
 							 const string& color,
 							 const string& link);
-
 schema* makeCableSchema 	(unsigned int n=1);
 schema* makeInverterSchema 	(const string& color);
 schema* makeCutSchema 		();
@@ -167,14 +166,7 @@ schema* makeSplitSchema 	(schema* s1, schema* s2);
 schema* makeRecSchema 		(schema* s1, schema* s2);
 schema* makeTopSchema 		(schema* s1, double margin, const string& text, const string& link);
 schema* makeDecorateSchema 	(schema* s1, double margin, const string& text);
-
-
-
-
-
-
-
-
+schema* makeConnectorSchema ();
 
 #endif
 
