@@ -19,12 +19,9 @@
  ************************************************************************
  ************************************************************************/
 
-
-
  /**********************************************************************
 			- klass.cpp : class C++ a remplir (projet FAUST) -
 
-
 		Historique :
 		-----------
 		17-10-2001 : implementation initiale (yo)
@@ -50,26 +47,8 @@
 #include "ppsig.hh"
 #include "recursivness.hh"
 
-
-extern bool gVectorSwitch;
-extern bool gDeepFirstSwitch;
-extern bool gOpenMPSwitch;
-extern bool gOpenMPLoop;
-extern bool gSchedulerSwitch;
-extern int  gVecSize;
-extern bool gUIMacroSwitch;
-extern int  gVectorLoopVariant;
-extern bool	gGroupTaskSwitch;
-
-extern map<Tree, set<Tree> > gMetaDataSet;
 static int gTaskCount = 0;
 
-void tab (int n, ostream& fout)
-{
-	fout << '\n';
-	while (n--)	fout << '\t';
-}
-
 bool Klass::fNeedPowerDef = false;
 
 /**
@@ -85,7 +64,7 @@ void Klass::setLoopProperty(Tree sig, Loop* l)
  */
 bool Klass::getLoopProperty(Tree sig, Loop*& l)
 {
-    return  fLoopProperty.get(sig, l);
+    return fLoopProperty.get(sig, l);
 }
 
 /**
@@ -115,10 +94,10 @@ void Klass::openLoop(Tree recsymbol, const string& size)
  */
 void Klass::closeLoop(Tree sig)
 {
-    assert(fTopLoop);
+    faustassert(fTopLoop);
     Loop* l = fTopLoop;
     fTopLoop = l->fEnclosingLoop;
-    assert(fTopLoop);
+    faustassert(fTopLoop);
 
     //l->println(4, cerr);
     //cerr << endl;
@@ -146,17 +125,6 @@ void Klass::closeLoop(Tree sig)
     //cerr << "\n" << endl;
 }
 
-/**
- * Print a list of lines.
- */
-void printlines(int n, list<string>& lines, ostream& fout)
-{
-    list<string>::iterator s;
-    for (s = lines.begin(); s != lines.end(); s++) {
-        tab(n, fout); fout << *s;
-    }
-}
-
 /**
  * Print a list of elements (e1, e2,...)
  */
@@ -200,13 +168,17 @@ void Klass::printIncludeFile(ostream& fout)
     set<string> S;
     set<string>::iterator f;
 
-    if (gOpenMPSwitch) {
+    if (gGlobal->gOpenMPSwitch) {
         fout << "#include <omp.h>" << "\n";
     }
 
     collectIncludeFile(S);
     for (f = S.begin(); f != S.end(); f++) 	{
-        fout << "#include " << *f << "\n";
+        string inc = *f;
+        // Only print non-empty include (inc is actually quoted)
+        if (inc.size() > 2) { 
+            fout << "#include " << *f << "\n";
+        }
     }
 }
 
@@ -217,27 +189,52 @@ void Klass::printAdditionalCode(ostream& fout)
 {
     if (fNeedPowerDef) {
         // Add faustpower definition to C++ code
+        fout << "#ifndef FAUSTPOWER" << endl;
+        fout << "#define FAUSTPOWER" << endl;
         fout << "#include <cmath>" << endl;
-        fout << "template <int N> inline float faustpower(float x) 		{ return powf(x,N); } " << endl;
-        fout << "template <int N> inline double faustpower(double x) 	{ return pow(x,N); }"  << endl;
-        fout << "template <int N> inline int faustpower(int x) 			{ return faustpower<N/2>(x) * faustpower<N-N/2>(x); } " << endl;
-        fout << "template <> 	 inline int faustpower<0>(int x) 		{ return 1; }" << endl;
-        fout << "template <> 	 inline int faustpower<1>(int x) 		{ return x; }" << endl;
-    }
+        
+        fout << "template <int N> inline int faustpower(int x)              { return faustpower<N/2>(x) * faustpower<N-N/2>(x); } " << endl;
+        fout << "template <> 	 inline int faustpower<0>(int x)            { return 1; }" << endl;
+        fout << "template <> 	 inline int faustpower<1>(int x)            { return x; }" << endl;
+        fout << "template <> 	 inline int faustpower<2>(int x)            { return x*x; }" << endl;
+        
+        if (gGlobal->gFloatSize == 1) {
+            
+            fout << "template <int N> inline float faustpower(float x)            { return faustpower<N/2>(x) * faustpower<N-N/2>(x); } " << endl;
+            fout << "template <> 	 inline float faustpower<0>(float x)          { return 1; }" << endl;
+            fout << "template <> 	 inline float faustpower<1>(float x)          { return x; }" << endl;
+            fout << "template <> 	 inline float faustpower<2>(float x)          { return x*x; }" << endl;
+            
+        } else if (gGlobal->gFloatSize == 2) {
+        
+            fout << "template <int N> inline double faustpower(double x)          { return faustpower<N/2>(x) * faustpower<N-N/2>(x); } " << endl;
+            fout << "template <> 	 inline double faustpower<0>(double x)        { return 1; }" << endl;
+            fout << "template <> 	 inline double faustpower<1>(double x)        { return x; }" << endl;
+            fout << "template <> 	 inline double faustpower<2>(double x)        { return x*x; }" << endl;
+            
+        } else if (gGlobal->gFloatSize == 3) {
+            
+            fout << "template <int N> inline long double faustpower(long double x){ return powl(x,N); }"  << endl;
+            
+        }
+        fout << "#endif" << endl;
 
+    }
 }
 
 /**
  * Print metadata declaration
  */
-void Klass::printMetadata(int n, const map<Tree, set<Tree> >& S, ostream& fout)
+void Klass::printMetadata(int n, const MetaDataSet& S, ostream& fout)
 {
-    tab(n,fout); fout   << "static void metadata(Meta* m) \t{ ";
-
-    for (map<Tree, set<Tree> >::iterator i = gMetaDataSet.begin(); i != gMetaDataSet.end(); i++) {
+    tab(n,fout); fout << "virtual void metadata(Meta* m) { ";
+    
+    // We do not want to accumulate metadata from all hierachical levels, so the upper level only is kept
+    for (MetaDataSet::iterator i = gGlobal->gMetaDataSet.begin(); i != gGlobal->gMetaDataSet.end(); i++) {
         if (i->first != tree("author")) {
             tab(n+1,fout); fout << "m->declare(\"" << *(i->first) << "\", " << **(i->second.begin()) << ");";
         } else {
+            // But the "author" meta data is accumulated, the upper level becomes the main author and sub-levels become "contributor"
             for (set<Tree>::iterator j = i->second.begin(); j != i->second.end(); j++) {
                 if (j == i->second.begin()) {
                      tab(n+1,fout); fout << "m->declare(\"" << *(i->first) << "\", " << **j << ");" ;
@@ -293,25 +290,28 @@ static void computeUseCount(Loop* l)
 /**
  * Group together sequences of loops
  */
-static void groupSeqLoops(Loop* l)
+static void groupSeqLoops(Loop* l, set<Loop*>& visited)
 {
-	int n = l->fBackwardLoopDependencies.size();
-	if (n==0) {
-		return;
-	} else if (n==1) {
-		Loop* f = *(l->fBackwardLoopDependencies.begin());
-		if (f->fUseCount ==  1) {
-			l->concat(f);
-			groupSeqLoops(l);
-		} else {
-			groupSeqLoops(f);
-		}
-		return;
-	} else if (n > 1) {
-		for (lset::iterator p =l->fBackwardLoopDependencies.begin(); p!=l->fBackwardLoopDependencies.end(); p++) {
-			groupSeqLoops(*p);
-		}
-	}
+    if (visited.find(l) == visited.end()) {
+        visited.insert(l);
+        int n = (int)l->fBackwardLoopDependencies.size();
+        if (n==0) {
+            return;
+        } else if (n==1) {
+            Loop* f = *(l->fBackwardLoopDependencies.begin());
+            if (f->fUseCount ==  1) {
+                l->concat(f);
+                groupSeqLoops(l, visited);
+            } else {
+                groupSeqLoops(f, visited);
+            }
+            return;
+        } else if (n > 1) {
+            for (lset::iterator p =l->fBackwardLoopDependencies.begin(); p!=l->fBackwardLoopDependencies.end(); p++) {
+                groupSeqLoops(*p, visited);
+            }
+        }
+    }
 }
 
 #define WORK_STEALING_INDEX 0
@@ -324,9 +324,10 @@ void Klass::buildTasksList()
 {
     lgraph G;
 
-    if (gGroupTaskSwitch) {
+    if (gGlobal->gGroupTaskSwitch) {
         computeUseCount(fTopLoop);
-        groupSeqLoops(fTopLoop);
+        set<Loop*> visited;
+        groupSeqLoops(fTopLoop, visited);
     }
 
     sortGraph(fTopLoop, G);
@@ -336,14 +337,14 @@ void Klass::buildTasksList()
     addDeclCode("FAUSTFLOAT** input;");
     addDeclCode("FAUSTFLOAT** output;");
     addDeclCode("volatile bool fIsFinished;");
-    addDeclCode("int fFullCount;");
+    addDeclCode("int fCount;");
     addDeclCode("int fIndex;");
     addDeclCode("DSPThreadPool* fThreadPool;");
     addDeclCode("int fStaticNumThreads;");
     addDeclCode("int fDynamicNumThreads;");
 
     // Compute forward dependencies
-    for (int l=G.size()-1; l>=0; l--) {
+    for (int l=(int)G.size()-1; l>=0; l--) {
         for (lset::const_iterator p =G[l].begin(); p!=G[l].end(); p++) {
             for (lset::const_iterator p1 = (*p)->fBackwardLoopDependencies.begin(); p1!=(*p)->fBackwardLoopDependencies.end(); p1++) {
                 (*p1)->fForwardLoopDependencies.insert((*p));
@@ -355,7 +356,7 @@ void Klass::buildTasksList()
 
     // Compute ready tasks list
     vector<int> task_num;
-    for (int l=G.size()-1; l>=0; l--) {
+    for (int l=(int)G.size()-1; l>=0; l--) {
         lset::const_iterator next;
         for (lset::const_iterator p =G[l].begin(); p!=G[l].end(); p++) {
             if ((*p)->fBackwardLoopDependencies.size() == 0) {
@@ -371,7 +372,7 @@ void Klass::buildTasksList()
         addZone3("if (cur_thread == 0) {");
 
         Loop* keep = NULL;
-        for (int l=G.size()-1; l>=0; l--) {
+        for (int l=(int)G.size()-1; l>=0; l--) {
             lset::const_iterator next;
             for (lset::const_iterator p =G[l].begin(); p!=G[l].end(); p++) {
                 if ((*p)->fBackwardLoopDependencies.size() == 0) {
@@ -419,7 +420,7 @@ void Klass::buildTasksList()
 
     // Compute init section
     addZone2c("// Only initialize taks with more than one input");
-    for (int l=G.size()-1; l>=0; l--) {
+    for (int l=(int)G.size()-1; l>=0; l--) {
         for (lset::const_iterator p =G[l].begin(); p!=G[l].end(); p++) {
             if ((*p)->fBackwardLoopDependencies.size() > 1)  { // Only initialize taks with more than 1 input, since taks with one input are "directly" activated.
                 addZone2c(subst("fGraph.InitTask($0,$1);", T(START_TASK_INDEX + gTaskCount++), T((int)(*p)->fBackwardLoopDependencies.size())));
@@ -431,7 +432,6 @@ void Klass::buildTasksList()
 
     addInitCode("fStaticNumThreads = get_max_cpu();");
     addInitCode("fDynamicNumThreads = getenv(\"OMP_NUM_THREADS\") ? atoi(getenv(\"OMP_NUM_THREADS\")) : fStaticNumThreads;");
-    addInitCode("fThreadPool = DSPThreadPool::Init();");
     addInitCode("fThreadPool->StartAll(fStaticNumThreads - 1, false);");
 
     gTaskCount = 0;
@@ -442,9 +442,10 @@ void Klass::buildTasksList()
  */
 void Klass::printLoopGraphVector(int n, ostream& fout)
 {
-    if (gGroupTaskSwitch) {
+    if (gGlobal->gGroupTaskSwitch) {
         computeUseCount(fTopLoop);
-        groupSeqLoops(fTopLoop);
+        set<Loop*> visited;
+        groupSeqLoops(fTopLoop, visited);
     }
 
     lgraph G;
@@ -452,7 +453,7 @@ void Klass::printLoopGraphVector(int n, ostream& fout)
 
 #if 1
     // EXPERIMENTAL
-    if (gVectorSwitch && gDeepFirstSwitch) {
+    if (gGlobal->gVectorSwitch && gGlobal->gDeepFirstSwitch) {
         set<Loop*> visited;
         printLoopDeepFirst(n, fout, fTopLoop, visited);
         return;
@@ -460,8 +461,8 @@ void Klass::printLoopGraphVector(int n, ostream& fout)
 #endif
 
     // normal mode
-    for (int l=G.size()-1; l>=0; l--) {
-        if (gVectorSwitch) { tab(n, fout); fout << "// SECTION : " << G.size() - l; }
+    for (int l=(int)G.size()-1; l>=0; l--) {
+        if (gGlobal->gVectorSwitch) { tab(n, fout); fout << "// SECTION : " << G.size() - l; }
         for (lset::const_iterator p =G[l].begin(); p!=G[l].end(); p++) {
             (*p)->println(n, fout);
         }
@@ -473,18 +474,19 @@ void Klass::printLoopGraphVector(int n, ostream& fout)
  */
 void Klass::printLoopGraphOpenMP(int n, ostream& fout)
 {
-    if (gGroupTaskSwitch) {
+    if (gGlobal->gGroupTaskSwitch) {
         computeUseCount(fTopLoop);
-        groupSeqLoops(fTopLoop);
+        set<Loop*> visited;
+        groupSeqLoops(fTopLoop, visited);
     }
 
     lgraph G;
     sortGraph(fTopLoop, G);
 
     // OpenMP mode : add OpenMP directives
-    for (int l=G.size()-1; l>=0; l--) {
+    for (int l=(int)G.size()-1; l>=0; l--) {
         tab(n, fout); fout << "// SECTION : " << G.size() - l;
-        printLoopLevelOpenMP(n, G.size() - l, G[l], fout);
+        printLoopLevelOpenMP(n, (int)G.size() - l, G[l], fout);
     }
 }
 
@@ -493,24 +495,24 @@ void Klass::printLoopGraphOpenMP(int n, ostream& fout)
  */
 void Klass::printLoopGraphScheduler(int n, ostream& fout)
 {
-    if (gGroupTaskSwitch) {
+    if (gGlobal->gGroupTaskSwitch) {
         computeUseCount(fTopLoop);
-        groupSeqLoops(fTopLoop);
+        set<Loop*> visited;
+        groupSeqLoops(fTopLoop, visited);
     }
 
     lgraph G;
     sortGraph(fTopLoop, G);
 
     // OpenMP mode : add OpenMP directives
-    for (int l=G.size()-1; l>0; l--) {
+    for (int l=(int)G.size()-1; l>0; l--) {
         tab(n, fout); fout << "// SECTION : " << G.size() - l;
-        printLoopLevelScheduler(n, G.size() - l, G[l], fout);
+        printLoopLevelScheduler(n, (int)G.size() - l, G[l], fout);
     }
 
-    printLastLoopLevelScheduler(n, G.size(), G[0], fout);
+    printLastLoopLevelScheduler(n, (int)G.size(), G[0], fout);
 }
 
-
 /**
  * Print the loop graph in dot format
  */
@@ -525,7 +527,7 @@ void Klass::printGraphDotFormat(ostream& fout)
 
     int lnum = 0;       // used for loop numbers
     // for each level of the graph
-    for (int l=G.size()-1; l>=0; l--) {
+    for (int l=(int)G.size()-1; l>=0; l--) {
         // for each task in the level
         for (lset::const_iterator t =G[l].begin(); t!=G[l].end(); t++) {
             // print task label "Lxxx : 0xffffff"
@@ -549,8 +551,8 @@ void Klass::printLoopGraphInternal(int n, ostream& fout)
     sortGraph(fTopLoop, G);
 
     // normal mode
-    for (int l=G.size()-1; l>=0; l--) {
-        if (gVectorSwitch) { tab(n, fout); fout << "// SECTION : " << G.size() - l; }
+    for (int l=(int)G.size()-1; l>=0; l--) {
+        if (gGlobal->gVectorSwitch) { tab(n, fout); fout << "// SECTION : " << G.size() - l; }
         for (lset::const_iterator p =G[l].begin(); p!=G[l].end(); p++) {
             (*p)->printoneln(n, fout);
         }
@@ -585,7 +587,7 @@ void Klass::printLoopLevelOpenMP(int n, int lnum, const lset& L, ostream& fout)
     if (nonRecursiveLevel(L) && L.size()==1) {
         for (lset::const_iterator p =L.begin(); p!=L.end(); p++) {
             if ((*p)->isEmpty() == false) {
-                if (gOpenMPLoop) {
+                if (gGlobal->gOpenMPLoop) {
                     (*p)->printParLoopln(n, fout);
                 } else {
                     tab(n, fout); fout << "#pragma omp single ";
@@ -733,15 +735,17 @@ void Klass::println(int n, ostream& fout)
 {
 	list<Klass* >::iterator k;
 
-    tab(n,fout); fout << "#define FAUSTCLASS "<< fKlassName << endl;
+    tab(n,fout); fout << "#ifndef FAUSTCLASS " << endl;
+    fout << "#define FAUSTCLASS "<< fKlassName << endl;
+    fout << "#endif" << endl;
 
-    if (gSchedulerSwitch) {
+    if (gGlobal->gSchedulerSwitch) {
         tab(n,fout); fout << "class " << fKlassName << " : public " << fSuperKlassName << ", public Runnable {";
     } else {
         tab(n,fout); fout << "class " << fKlassName << " : public " << fSuperKlassName << " {";
     }
 
-    if (gUIMacroSwitch) {
+    if (gGlobal->gUIMacroSwitch) {
         tab(n,fout); fout << "  public:";
     } else {
 	    tab(n,fout); fout << "  private:";
@@ -750,41 +754,81 @@ void Klass::println(int n, ostream& fout)
     for (k = fSubClassList.begin(); k != fSubClassList.end(); k++) 	(*k)->println(n+1, fout);
 
     printlines(n+1, fDeclCode, fout);
+    
+    tab(n+1,fout); fout << "int fSamplingFreq;\n";
 
-	tab(n,fout); fout << "  public:";
-
-    printMetadata(n+1, gMetaDataSet, fout);
-
-    if (gSchedulerSwitch) {
-        tab(n+1,fout); fout << "virtual ~" << fKlassName << "() \t{ "
-                            << "DSPThreadPool::Destroy()"
-                            << "; }";
+    tab(n,fout); fout << "  public:";
+    
+    if (gGlobal->gMemoryManager) {
+        tab(n+1,fout); fout << "static dsp_memory_manager* fManager;" << endl;
     }
 
-    tab(n+1,fout); fout     << "virtual int getNumInputs() \t{ "
+    printMetadata(n+1, gGlobal->gMetaDataSet, fout);
+
+    if (gGlobal->gSchedulerSwitch) {
+        tab(n+1,fout); fout << fKlassName << "() { "
+                            << "fThreadPool = DSPThreadPool::Init(); }";
+        
+        tab(n+1,fout); fout << "virtual ~" << fKlassName << "() { "
+                            << "DSPThreadPool::Destroy(); }";
+    }
+    
+    tab(n+1,fout); fout << "virtual int getNumInputs() { "
                     << "return " << fNumInputs
                     << "; }";
-    tab(n+1,fout); fout 	<< "virtual int getNumOutputs() \t{ "
+    
+    tab(n+1,fout); fout << "virtual int getNumOutputs() { "
                     << "return " << fNumOutputs
                     << "; }";
 
     tab(n+1,fout); fout << "static void classInit(int samplingFreq) {";
-        printlines (n+2, fStaticInitCode, fout);
+        printlines(n+2, fStaticInitCode, fout);
     tab(n+1,fout); fout << "}";
+    
+    if (gGlobal->gMemoryManager) {
+        tab(n+1,fout); fout << "static void classDestroy() {";
+            printlines(n+2, fStaticDestroyCode, fout);
+        tab(n+1,fout); fout << "}";
+    }
 
-    tab(n+1,fout); fout << "virtual void instanceInit(int samplingFreq) {";
+    tab(n+1,fout); fout << "virtual void instanceConstants(int samplingFreq) {";
         tab(n+2,fout); fout << "fSamplingFreq = samplingFreq;";
-        printlines (n+2, fInitCode, fout);
+        printlines(n+2, fInitCode, fout);
+    tab(n+1,fout); fout << "}";
+    
+    tab(n+1,fout); fout << "virtual void instanceResetUserInterface() {";
+        printlines(n+2, fInitUICode, fout);
+    tab(n+1,fout); fout << "}";
+    
+    tab(n+1,fout); fout << "virtual void instanceClear() {";
+        printlines(n+2, fClearCode, fout);
     tab(n+1,fout); fout << "}";
 
-    tab(n+1,fout); fout << "virtual void init(int samplingFreq) {";
-        tab(n+2,fout); fout << "classInit(samplingFreq);";
-         tab(n+2,fout); fout << "instanceInit(samplingFreq);";
+    if (gGlobal->gMemoryManager) {
+        tab(n+1,fout); fout << "virtual void init(int samplingFreq) {}";
+    } else {
+        tab(n+1,fout); fout << "virtual void init(int samplingFreq) {";
+            tab(n+2,fout); fout << "classInit(samplingFreq);";
+            tab(n+2,fout); fout << "instanceInit(samplingFreq);";
+        tab(n+1,fout); fout << "}";
+    }
+    
+    tab(n+1,fout); fout << "virtual void instanceInit(int samplingFreq) {";
+        tab(n+2,fout); fout << "instanceConstants(samplingFreq);";
+        tab(n+2,fout); fout << "instanceResetUserInterface();";
+        tab(n+2,fout); fout << "instanceClear();";
+    tab(n+1,fout); fout << "}";
+    
+    tab(n+1,fout); fout << "virtual "<< fKlassName <<"* clone() {";
+        tab(n+2,fout); fout << "return new " << fKlassName << "();";
+    tab(n+1,fout); fout << "}";
+    
+    tab(n+1,fout); fout << "virtual int getSampleRate() {";
+        tab(n+2,fout); fout << "return fSamplingFreq;";
     tab(n+1,fout); fout << "}";
 
-
-    tab(n+1,fout); fout << "virtual void buildUserInterface(UI* interface) {";
-        printlines (n+2, fUICode, fout);
+    tab(n+1,fout); fout << "virtual void buildUserInterface(UI* ui_interface) {";
+        printlines(n+2, fUICode, fout);
     tab(n+1,fout); fout << "}";
 
     printComputeMethod(n, fout);
@@ -792,9 +836,13 @@ void Klass::println(int n, ostream& fout)
 	tab(n,fout); fout << "};\n" << endl;
 
 	printlines(n, fStaticFields, fout);
-
+    
+    if (gGlobal->gMemoryManager) {
+        tab(n, fout); fout << "dsp_memory_manager* " << fKlassName <<"::fManager = 0;" << endl;
+    }
+    
 	// generate user interface macros if needed
-	if (gUIMacroSwitch) {
+	if (gGlobal->gUIMacroSwitch) {
 		tab(n, fout); fout << "#ifdef FAUST_UIMACROS";
             tab(n+1,fout); fout << "#define FAUST_INPUTS " << fNumInputs;
             tab(n+1,fout); fout << "#define FAUST_OUTPUTS " << fNumOutputs;
@@ -812,15 +860,15 @@ void Klass::println(int n, ostream& fout)
  */
 void Klass::printComputeMethod(int n, ostream& fout)
 {
-    if (gSchedulerSwitch) {
+    if (gGlobal->gSchedulerSwitch) {
         printComputeMethodScheduler (n, fout);
-    } else if (gOpenMPSwitch) {
+    } else if (gGlobal->gOpenMPSwitch) {
         printComputeMethodOpenMP (n, fout);
-    } else if (gVectorSwitch) {
-        switch (gVectorLoopVariant) {
+    } else if (gGlobal->gVectorSwitch) {
+        switch (gGlobal->gVectorLoopVariant) {
             case 0 : printComputeMethodVectorFaster(n, fout); break;
             case 1 : printComputeMethodVectorSimple(n, fout); break;
-            default : cerr << "unknown loop variant " << gVectorLoopVariant << endl; exit(1);
+            default : cerr << "unknown loop variant " << gGlobal->gVectorLoopVariant << endl; exit(1);
         }
    } else {
         printComputeMethodScalar(n, fout);
@@ -830,11 +878,17 @@ void Klass::printComputeMethod(int n, ostream& fout)
 void Klass::printComputeMethodScalar(int n, ostream& fout)
 {
     tab(n+1,fout); fout << subst("virtual void compute (int count, $0** input, $0** output) {", xfloat());
-        printlines (n+2, fZone1Code, fout);
-        printlines (n+2, fZone2Code, fout);
-        printlines (n+2, fZone2bCode, fout);
-        printlines (n+2, fZone3Code, fout);
+        tab(n+2,fout); fout << "//zone1";
+        printlines(n+2, fZone1Code, fout);
+        tab(n+2,fout); fout << "//zone2";
+        printlines(n+2, fZone2Code, fout);
+        tab(n+2,fout); fout << "//zone2b";
+        printlines(n+2, fZone2bCode, fout);
+        tab(n+2,fout); fout << "//zone3";
+        printlines(n+2, fZone3Code, fout);
+        tab(n+2,fout); fout << "//LoopGraphScalar";
         printLoopGraphScalar (n+2,fout);
+        printlines(n+2, fZone4Code, fout);
     tab(n+1,fout); fout << "}";
 }
 
@@ -847,25 +901,29 @@ void Klass::printComputeMethodVectorFaster(int n, ostream& fout)
 {
     // in vector mode we need to split loops in smaller pieces not larger
     // than gVecSize
-    tab(n+1,fout); fout << subst("virtual void compute (int fullcount, $0** input, $0** output) {", xfloat());
+    tab(n+1,fout); fout << subst("virtual void compute (int count, $0** input, $0** output) {", xfloat());
         printlines(n+2, fZone1Code, fout);
         printlines(n+2, fZone2Code, fout);
         printlines(n+2, fZone2bCode, fout);
 
         tab(n+2,fout); fout << "int index;";
-        tab(n+2,fout); fout << "for (index = 0; index <= fullcount - " << gVecSize << "; index += " << gVecSize << ") {";
-            tab(n+3,fout); fout << "// compute by blocks of " << gVecSize << " samples";
-            tab(n+3,fout); fout << "const int count = " << gVecSize << ";";
-            printlines (n+3, fZone3Code, fout);
+		tab(n+2,fout); fout << "int fullcount = count;";
+		
+        tab(n+2,fout); fout << "for (index = 0; index <= fullcount - " << gGlobal->gVecSize << "; index += " << gGlobal->gVecSize << ") {";
+            tab(n+3,fout); fout << "// compute by blocks of " << gGlobal->gVecSize << " samples";
+            tab(n+3,fout); fout << "const int count = " << gGlobal->gVecSize << ";";
+            printlines(n+3, fZone3Code, fout);
             printLoopGraphVector(n+3,fout);
         tab(n+2,fout); fout << "}";
 
         tab(n+2,fout); fout << "if (index < fullcount) {";
             tab(n+3,fout); fout << "// compute the remaining samples if any";
             tab(n+3,fout); fout << "int count = fullcount-index;";
-            printlines (n+3, fZone3Code, fout);
+            printlines(n+3, fZone3Code, fout);
              printLoopGraphVector(n+3,fout);
         tab(n+2,fout); fout << "}";
+
+        printlines(n+2, fZone4Code, fout);
     tab(n+1,fout); fout << "}";
 }
 
@@ -876,15 +934,19 @@ void Klass::printComputeMethodVectorSimple(int n, ostream& fout)
 {
     // in vector mode we need to split loops in smaller pieces not larger
     // than gVecSize
-    tab(n+1,fout); fout << subst("virtual void compute (int fullcount, $0** input, $0** output) {", xfloat());
+    tab(n+1,fout); fout << subst("virtual void compute (int count, $0** input, $0** output) {", xfloat());
         printlines(n+2, fZone1Code, fout);
         printlines(n+2, fZone2Code, fout);
         printlines(n+2, fZone2bCode, fout);
-        tab(n+2,fout); fout << "for (int index = 0; index < fullcount; index += " << gVecSize << ") {";
-            tab(n+3,fout); fout << "int count = min("<< gVecSize << ", fullcount-index);";
-            printlines (n+3, fZone3Code, fout);
+		
+        tab(n+2,fout); fout << "int fullcount = count;";
+		tab(n+2,fout); fout << "for (int index = 0; index < fullcount; index += " << gGlobal->gVecSize << ") {";
+            tab(n+3,fout); fout << "int count = min("<< gGlobal->gVecSize << ", fullcount-index);";
+            printlines(n+3, fZone3Code, fout);
             printLoopGraphVector(n+3,fout);
         tab(n+2,fout); fout << "}";
+
+        printlines(n+2, fZone4Code, fout);
     tab(n+1,fout); fout << "}";
 }
 
@@ -947,9 +1009,10 @@ void Klass::printComputeMethodOpenMP(int n, ostream& fout)
 {
     // in openMP mode we need to split loops in smaller pieces not larger
     // than gVecSize and add OpenMP pragmas
-    tab(n+1,fout); fout << subst("virtual void compute (int fullcount, $0** input, $0** output) {", xfloat());
+    tab(n+1,fout); fout << subst("virtual void compute (int count, $0** input, $0** output) {", xfloat());
         printlines(n+2, fZone1Code, fout);
         printlines(n+2, fZone2Code, fout);
+        tab(n+2,fout); fout << "int fullcount = count;";
         tab(n+2,fout); fout << "#pragma omp parallel";
         printdecllist(n+3, "firstprivate", fFirstPrivateDecl, fout);
 
@@ -961,15 +1024,17 @@ void Klass::printComputeMethodOpenMP(int n, ostream& fout)
                 tab(n+3,fout); fout << "}";
             }
 
-            tab(n+3,fout); fout << "for (int index = 0; index < fullcount; index += " << gVecSize << ") {";
-            tab(n+4,fout); fout << "int count = min ("<< gVecSize << ", fullcount-index);";
+            tab(n+3,fout); fout << "for (int index = 0; index < fullcount; index += " << gGlobal->gVecSize << ") {";
+            tab(n+4,fout); fout << "int count = min ("<< gGlobal->gVecSize << ", fullcount-index);";
 
-            printlines (n+4, fZone3Code, fout);
-            printLoopGraphOpenMP (n+4,fout);
+            printlines(n+4, fZone3Code, fout);
+            printLoopGraphOpenMP(n+4,fout);
 
             tab(n+3,fout); fout << "}";
 
         tab(n+2,fout); fout << "}";
+
+        printlines(n+2, fZone4Code, fout);
     tab(n+1,fout); fout << "}";
 }
 
@@ -977,13 +1042,13 @@ void Klass::printComputeMethodOpenMP(int n, ostream& fout)
 void Klass::printComputeMethodScheduler (int n, ostream& fout)
 {
     tab(n+1,fout); fout << subst("virtual void compute (int fullcount, $0** input, $0** output) {", xfloat());
-        printlines (n+2, fZone1Code, fout);
-        printlines (n+2, fZone2Code, fout);
+        printlines(n+2, fZone1Code, fout);
+        printlines(n+2, fZone2Code, fout);
 
         // Init input and output
         tab(n+2,fout); fout << "// Init input and output";
-        printlines (n+2, fZone3aCode, fout);
-        printlines (n+2, fZone3bCode, fout);
+        printlines(n+2, fZone3aCode, fout);
+        printlines(n+2, fZone3bCode, fout);
 
         tab(n+2,fout); fout << "// Init graph state";
         tab(n+2,fout); fout << "initState(fTasksList);";
@@ -1010,7 +1075,7 @@ void Klass::printComputeMethodScheduler (int n, ostream& fout)
 
                             // Input task
                             tab(n+7, fout); fout << "case " << gTaskCount++ << ": { ";
-                            printlines (n+8, fZone6Code, fout);
+                            printlines(n+8, fZone6Code, fout);
                             tab(n+8, fout); fout << "index_in += count;";
                             tab(n+8, fout); fout << "last_cycle_for_thread = (index_in > fullcount);";
                             tab(n+8, fout); fout << "break;";
@@ -1018,7 +1083,7 @@ void Klass::printComputeMethodScheduler (int n, ostream& fout)
 
                             // Output task
                             tab(n+7, fout); fout << "case " << gTaskCount++ << ": { ";
-                            printlines (n+8, fZone7Code, fout);
+                            printlines(n+8, fZone7Code, fout);
                             tab(n+8, fout); fout << "index_out += count;";
                             tab(n+8, fout); fout << "last_cycle_for_thread = (index_out > fullcount);";
                             tab(n+8, fout); fout << "break;";
@@ -1048,7 +1113,7 @@ void Klass::printComputeMethodScheduler (int n, ostream& fout)
         tab(n+2,fout); fout << "fGraph.Display();";
     tab(n+1,fout); fout << "}";
 
-    tab(n+1,fout); fout << subst("virtual void compute (int fullcount, $0** input, $0** output) {", xfloat());
+    tab(n+1,fout); fout << subst("virtual void compute (int count, $0** input, $0** output) {", xfloat());
 
         tab(n+2,fout); fout << "GetRealTime();";
 
@@ -1056,12 +1121,14 @@ void Klass::printComputeMethodScheduler (int n, ostream& fout)
         tab(n+2,fout); fout << "this->output = output;";
 
         tab(n+2,fout); fout << "StartMeasure();";
+        
+        tab(n+2,fout); fout << "int fullcount = count;";
 
-        tab(n+2,fout); fout << "for (fIndex = 0; fIndex < fullcount; fIndex += " << gVecSize << ") {";
+        tab(n+2,fout); fout << "for (fIndex = 0; fIndex < fullcount; fIndex += " << gGlobal->gVecSize << ") {";
 
-        tab(n+3,fout); fout << "fFullCount = min ("<< gVecSize << ", fullcount-fIndex);";
+        tab(n+3,fout); fout << "fCount = min ("<< gGlobal->gVecSize << ", fullcount-fIndex);";
         tab(n+3,fout); fout << "TaskQueue::Init();";
-        printlines (n+3, fZone2cCode, fout);
+        printlines(n+3, fZone2cCode, fout);
 
         tab(n+3,fout); fout << "fIsFinished = false;";
         tab(n+3,fout); fout << "fThreadPool->SignalAll(fDynamicNumThreads - 1, this);";
@@ -1075,19 +1142,21 @@ void Klass::printComputeMethodScheduler (int n, ostream& fout)
     tab(n+1,fout); fout << "}";
 
     tab(n+1,fout); fout << "void computeThread(int cur_thread) {";
-        printlines (n+2, fZone1Code, fout);
-        printlines (n+2, fZone2Code, fout);
+    
+        tab(n+2,fout); fout << "int count = fCount;";
+        
+        printlines(n+2, fZone1Code, fout);
+        printlines(n+2, fZone2Code, fout);
 
         tab(n+2,fout); fout << "// Init graph state";
 
         tab(n+2,fout); fout << "{";
             tab(n+3,fout); fout << "TaskQueue taskqueue(cur_thread);";
             tab(n+3,fout); fout << "int tasknum = -1;";
-            tab(n+3,fout); fout << "int count = fFullCount;";
-
+    
             // Init input and output
             tab(n+3,fout); fout << "// Init input and output";
-            printlines (n+3, fZone3Code, fout);
+            printlines(n+3, fZone3Code, fout);
 
             tab(n+3,fout); fout << "while (!fIsFinished) {";
                  tab(n+4,fout); fout << "switch (tasknum) {";
@@ -1107,7 +1176,7 @@ void Klass::printComputeMethodScheduler (int n, ostream& fout)
                     gTaskCount = START_TASK_INDEX;
 
                     // DSP tasks
-                    printLoopGraphScheduler (n+5,fout);
+                    printLoopGraphScheduler(n+5,fout);
 
                  tab(n+4,fout); fout << "}";
             tab(n+3,fout); fout << "}";
@@ -1125,7 +1194,7 @@ void SigIntGenKlass::println(int n, ostream& fout)
 	tab(n,fout); fout << "class " << fKlassName << " {";
 
 	tab(n,fout); fout << "  private:";
-		tab(n+1,fout); fout << "int \tfSamplingFreq;";
+		tab(n+1,fout); fout << "int fSamplingFreq;";
 
 		for (k = fSubClassList.begin(); k != fSubClassList.end(); k++) 	(*k)->println(n+1, fout);
 
@@ -1133,20 +1202,22 @@ void SigIntGenKlass::println(int n, ostream& fout)
 
 	tab(n,fout); fout << "  public:";
 
-		tab(n+1,fout); fout 	<< "int getNumInputs() \t{ "
+		tab(n+1,fout); fout << "int getNumInputs() { "
 						<< "return " << fNumInputs << "; }";
-		tab(n+1,fout); fout 	<< "int getNumOutputs() \t{ "
+		tab(n+1,fout); fout << "int getNumOutputs() { "
 						<< "return " << fNumOutputs << "; }";
 
 		tab(n+1,fout); fout << "void init(int samplingFreq) {";
 			tab(n+2,fout); fout << "fSamplingFreq = samplingFreq;";
+            printlines(n+2, fInitCode, fout);
+            printlines(n+2, fClearCode, fout);
 		tab(n+1,fout); fout << "}";
 
 		tab(n+1,fout); fout << "void fill (int count, int output[]) {";
-            printlines (n+2, fZone1Code, fout);
-            printlines (n+2, fZone2Code, fout);
-            printlines (n+2, fZone2bCode, fout);
-            printlines (n+2, fZone3Code, fout);
+            printlines(n+2, fZone1Code, fout);
+            printlines(n+2, fZone2Code, fout);
+            printlines(n+2, fZone2bCode, fout);
+            printlines(n+2, fZone3Code, fout);
             printLoopGraphInternal (n+2,fout);
 		tab(n+1,fout); fout << "}";
 
@@ -1163,7 +1234,7 @@ void SigFloatGenKlass::println(int n, ostream& fout)
 	tab(n,fout); fout << "class " << fKlassName << " {";
 
 	tab(n,fout); fout << "  private:";
-		tab(n+1,fout); fout << "int \tfSamplingFreq;";
+		tab(n+1,fout); fout << "int fSamplingFreq;";
 
 		for (k = fSubClassList.begin(); k != fSubClassList.end(); k++) 	(*k)->println(n+1, fout);
 
@@ -1171,21 +1242,22 @@ void SigFloatGenKlass::println(int n, ostream& fout)
 
 	tab(n,fout); fout << "  public:";
 
-		tab(n+1,fout); fout 	<< "int getNumInputs() \t{ "
+		tab(n+1,fout); fout << "int getNumInputs() { "
 						<< "return " << fNumInputs << "; }";
-		tab(n+1,fout); fout 	<< "int getNumOutputs() \t{ "
+		tab(n+1,fout); fout << "int getNumOutputs() { "
 						<< "return " << fNumOutputs << "; }";
 
 		tab(n+1,fout); fout << "void init(int samplingFreq) {";
 			tab(n+2,fout); fout << "fSamplingFreq = samplingFreq;";
 			printlines(n+2, fInitCode, fout);
+            printlines(n+2, fClearCode, fout);
 		tab(n+1,fout); fout << "}";
 
 		tab(n+1,fout); fout << subst("void fill (int count, $0 output[]) {", ifloat());
-            printlines (n+2, fZone1Code, fout);
-            printlines (n+2, fZone2Code, fout);
-            printlines (n+2, fZone2bCode, fout);
-            printlines (n+2, fZone3Code, fout);
+            printlines(n+2, fZone1Code, fout);
+            printlines(n+2, fZone2Code, fout);
+            printlines(n+2, fZone2bCode, fout);
+            printlines(n+2, fZone3Code, fout);
             printLoopGraphInternal(n+2,fout);
  		tab(n+1,fout); fout << "}";
 
@@ -1200,7 +1272,7 @@ static void merge (set<string>& dst, set<string>& src)
 
 void Klass::collectIncludeFile(set<string>& S)
 {
-	list<Klass* >::iterator 	k;
+	list<Klass* >::iterator k;
 
 	for (k = fSubClassList.begin(); k != fSubClassList.end(); k++) 	(*k)->collectIncludeFile(S);
 	merge(S, fIncludeFileSet);
@@ -1208,7 +1280,7 @@ void Klass::collectIncludeFile(set<string>& S)
 
 void Klass::collectLibrary(set<string>& S)
 {
-	list<Klass* >::iterator 	k;
+	list<Klass* >::iterator k;
 
 	for (k = fSubClassList.begin(); k != fSubClassList.end(); k++) 	(*k)->collectLibrary(S);
 	merge(S, fLibrarySet);
