@@ -1,7 +1,7 @@
 /************************************************************************
  ************************************************************************
     FAUST compiler
-	Copyright (C) 2003-2004 GRAME, Centre National de Creation Musicale
+    Copyright (C) 2003-2004 GRAME, Centre National de Creation Musicale
     ---------------------------------------------------------------------
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,24 +19,21 @@
  ************************************************************************
  ************************************************************************/
  
- 
- 
+#include <sstream>
 #include "uitree.hh"
-
-
+#include "exception.hh"
+#include "global.hh"
 
 static Tree makeSubFolderChain(Tree path, Tree elem);
 static Tree putFolder(Tree folder, Tree item);
 static Tree getFolder (Tree folder, Tree ilabel);
 
-
 static void error(const char * s, Tree t)
 {
-	fprintf(stderr, "ERROR : %s (%p)\n", s, t);
+	fprintf(stderr, "ERROR : %s (%p)\n", s, (void*)t);
 }
 
-#define ERROR(s,t) error(s,t); exit(1)
-
+#define FAUST_ERROR(s,t) { error(s,t); throw faustexception(s); }
 
 //------------------------------------------------------------------------------
 // Property list
@@ -53,14 +50,14 @@ static bool findKey (Tree pl, Tree key, Tree& val)
 
 static Tree updateKey (Tree pl, Tree key, Tree val)
 {
-	if (isNil(pl)) 				return cons ( cons(key,val), nil );
+	if (isNil(pl)) 				return cons ( cons(key,val), gGlobal->nil );
 	if (left(hd(pl)) == key) 	return cons ( cons(key,val), tl(pl) );
 	/*  left(hd(pl)) != key	*/	return cons ( hd(pl), updateKey( tl(pl), key, val ));
 }
 
 static Tree removeKey (Tree pl, Tree key)
 {
-	if (isNil(pl)) 				return nil;
+	if (isNil(pl)) 				return gGlobal->nil;
 	if (left(hd(pl)) == key) 	return tl(pl);
 	/*  left(hd(pl)) != key	*/	return cons (hd(pl), removeKey(tl(pl), key));
 }
@@ -78,17 +75,16 @@ static bool isBefore(Tree k1, Tree k2)
 	//fprintf(stderr, "isBefore("); print(k1, stderr); fprintf(stderr,", "); print(k2, stderr); fprintf(stderr,")\n"); 
 	Sym s1, s2;
 	if (!isSym(k1->node(), &s1)) {
-		ERROR("the node of the tree is not a symbol", k1);
+		FAUST_ERROR("the node of the tree is not a symbol", k1);
 	}
 	if (!isSym(k2->node(), &s2)) {
-		ERROR("the node of the tree is not a symbol", k2);
+		FAUST_ERROR("the node of the tree is not a symbol", k2);
 	}
 	
 	//fprintf (stderr, "strcmp(\"%s\", \"%s\") = %d\n", name(s1), name(s2), strcmp(name(s1), name(s2)));
 	return strcmp(name(s1), name(s2)) < 0;
 }
 
-
 static bool findKey (Tree pl, Tree key, Tree& val)
 {
 	if (isNil(pl)) 					return false;
@@ -99,7 +95,7 @@ static bool findKey (Tree pl, Tree key, Tree& val)
 
 static Tree updateKey (Tree pl, Tree key, Tree val)
 {
-    if (isNil(pl))                  return cons ( cons(key,val), nil );
+    if (isNil(pl))                  return cons ( cons(key,val), gGlobal->nil );
     if (left(hd(pl)) == key)        return cons ( cons(key,val), tl(pl) );
     if (isBefore(left(hd(pl)),key)) return cons ( hd(pl), updateKey( tl(pl), key, val ));
     return cons(cons(key,val), pl);
@@ -110,16 +106,15 @@ static Tree updateKey (Tree pl, Tree key, Tree val)
  */
 static Tree addKey (Tree pl, Tree key, Tree val)
 {
-    if (isNil(pl))                  return cons ( cons(key,val), nil );
+    if (isNil(pl))                  return cons ( cons(key,val), gGlobal->nil );
     if (isBefore(key, left(hd(pl)))) return cons(cons(key,val), pl);
     return cons ( hd(pl), addKey( tl(pl), key, val ));
 }
 
-
 #if 0
 static Tree removeKey (Tree pl, Tree key)
 {
-	if (isNil(pl)) 					return nil;
+	if (isNil(pl)) 					return gGlobal->nil;
 	if (left(hd(pl)) == key) 		return tl(pl);
 	if (isBefore(left(hd(pl)),key))	return cons (hd(pl), removeKey(tl(pl), key));
 	return pl;
@@ -131,16 +126,12 @@ static Tree removeKey (Tree pl, Tree key)
 // gestion de la construction de l'arbre d'interface utilisateur
 //------------------------------------------------------------------------------
 
-Sym 	UIFOLDER = symbol ("uiFolder");
-Tree  	uiFolder(Tree label, Tree elements)				{ return tree(UIFOLDER, label, elements); 		}
-bool  	isUiFolder(Tree t)								{ return isTree(t, UIFOLDER); 					}
-bool  	isUiFolder(Tree t, Tree& label, Tree& elements)	{ return isTree(t, UIFOLDER, label, elements); 	}
-
-Sym 	UIWIDGET = symbol ("uiWidget");
-Tree 	uiWidget(Tree label, Tree varname, Tree sig) 					{ return tree(UIWIDGET, label, varname, sig); }
-bool 	isUiWidget(Tree t, Tree& label, Tree& varname, Tree& sig)		{ return isTree(t, UIWIDGET, label, varname, sig); }
-
+Tree  	uiFolder(Tree label, Tree elements)				{ return tree(gGlobal->UIFOLDER, label, elements); 		}
+bool  	isUiFolder(Tree t)								{ return isTree(t, gGlobal->UIFOLDER); 					}
+bool  	isUiFolder(Tree t, Tree& label, Tree& elements)	{ return isTree(t, gGlobal->UIFOLDER, label, elements); 	}
 
+Tree 	uiWidget(Tree label, Tree varname, Tree sig) 					{ return tree(gGlobal->UIWIDGET, label, varname, sig); }
+bool 	isUiWidget(Tree t, Tree& label, Tree& varname, Tree& sig)		{ return isTree(t, gGlobal->UIWIDGET, label, varname, sig); }
 
 // place un item dans un folder. Remplace eventuellement l'lment de mme nom.
 Tree putFolder(Tree folder, Tree item)
@@ -168,7 +159,7 @@ Tree getFolder (Tree folder, Tree ilabel)
 	if (findKey(content, ilabel, item)) {
 		return item;
 	} else {
-		return nil;
+		return gGlobal->nil;
 	}
 }
 	
@@ -178,10 +169,9 @@ Tree makeSubFolderChain(Tree path, Tree elem)
 	if (isNil(path)) {
 		return elem;
 	} else {
-		return putFolder(uiFolder(hd(path)), makeSubFolderChain(tl(path),elem));
+		return putFolder(uiFolder(hd(path), gGlobal->nil), makeSubFolderChain(tl(path),elem));
 	}
 } 
-
 	
 Tree putSubFolder(Tree folder, Tree path, Tree item) 
 {
@@ -197,19 +187,30 @@ Tree putSubFolder(Tree folder, Tree path, Tree item)
 		}
 	}
 }
-
 	
 /*
 Fonctionnement des dossiers. 
-Dossier  1 niveau : Un dossier contient une liste de choses reperes par un nom  :
+Dossier a 1 niveau : Un dossier contient une liste de choses reperees par un nom :
 	Dossier[(l1,d1)...(ln,dn)] 
-ou (lx,dx) est une chose dx repre par un nom lx. On suppose les lx tous diffrents
+ou (lx,dx) est une chose dx reperee par un nom lx. On suppose les lx tous differents
 
-On peut ajouter une chose  un dossier : Ajouter(Dossier, Chose) -> Dossier
+On peut ajouter une chose a un dossier : Ajouter(Dossier, Chose) -> Dossier
 
-Si le dossier contient deja qq chose de meme nom, cette chose est remplace par la nouvelle.
+Si le dossier contient deja qq chose de meme nom, cette chose est remplacee par la nouvelle.
 
 AJOUTER (Dossier[(l1,d1)...(ln,dn)], (lx,dx)) -> Dossier[(l1,d1)...(lx,dx)...(ln,dn)]
 
 AJOUTER (Dossier[(l1,d1)...(lx,dx)...(ln,dn)], (lx,dx')) -> Dossier[(l1,d1)...(lx,dx')...(ln,dn)]
 */
+
+
+// Handle empty labels in a consistent way
+string ptrToHex(Tree ptr)
+{
+    stringstream res; res << hex << ptr; return res.str();
+}
+
+string checkNullLabel(Tree t, const string& label, bool bargraph)
+{
+    return (label == "") ? (bargraph ? ptrToHex(t) : string("0x00")) : label;
+}
