@@ -19,13 +19,16 @@
  ************************************************************************
  ************************************************************************/
 
-
-
+#include "Text.hh"
 #include "ppsig.hh"
 #include "binop.hh"
 #include "prim2.hh"
 #include "xtended.hh"
 #include "recursivness.hh"
+#include "exception.hh"
+#include "global.hh"
+
+ppsig::ppsig(Tree s) : sig(s), fEnv(gGlobal->nil), fPriority(0), fHideRecursion(false)  {}
 
 ostream& ppsig::printinfix (ostream& fout, const string& opname, int priority, Tree x, Tree y) const
 {
@@ -173,13 +176,12 @@ ostream& ppsig::printextended (ostream& fout, Tree sig) const
 	fout << ')';
 	return fout;
 }
-	
-	
+		
 ostream& ppsig::print (ostream& fout) const
 {
 	int 	i;
 	double	r;
-    Tree 	c, sel, x, y, z, u, var, le, label, id, ff, largs, type, name, file;
+    Tree 	c, sel, x, y, z, u, var, le, label, id, ff, largs, type, name, file, sf;
 
 		  if ( isList(sig) ) 						{ printlist(fout, sig); }
 	else if ( isProj(sig, &i, x) ) 					{ fout << "proj" << i << '(' << ppsig(x, fEnv) << ')';	}
@@ -191,8 +193,9 @@ ostream& ppsig::print (ostream& fout) const
 	
 	else if ( getUserData(sig) ) 					{ printextended(fout, sig); }
 	else if ( isSigInt(sig, &i) ) 					{ fout << i; }
-	else if ( isSigReal(sig, &r) ) 					{ fout << r; }
-	else if ( isSigInput(sig, &i) ) 				{ fout << "IN[" << i << "]"; }
+    else if ( isSigReal(sig, &r) ) 					{ fout << T(r); }
+    else if ( isSigWaveform(sig) )                  { fout << "waveform{...}"; }
+    else if ( isSigInput(sig, &i) ) 				{ fout << "IN[" << i << "]"; }
 	else if ( isSigOutput(sig, &i, x) ) 			{ printout(fout, i, x) ; }
 	
 	else if ( isSigDelay1(sig, x) ) 				{ fout << ppsig(x, fEnv, 9) << "'"; }
@@ -227,11 +230,20 @@ ostream& ppsig::print (ostream& fout) const
 	else if ( isSigNumEntry(sig, label,c,x,y,z) )	{ printui(fout, "nentry", label, c, x, y, z); }
 	else if ( isSigVBargraph(sig, label,x,y,z) )	{ printui(fout, "vbargraph", label, x, y, z); }
 	else if ( isSigHBargraph(sig, label,x,y,z) )	{ printui(fout, "hbargraph", label, x, y, z); }
+
+	else if (isSigSoundfile(sig, label) )			{ printui(fout, "soundfile", label); }
+	else if (isSigSoundfileLength(sig, sf) )		{ printfun(fout, "length", sf); }
+	else if (isSigSoundfileRate(sig, sf) )	    	{ printfun(fout, "rate", sf); }
+	else if (isSigSoundfileChannel(sig,sf,x,y)) 	{ printfun(fout, "channel", sf, x, y); }
+
 	else if ( isSigAttach(sig, x, y) )				{ printfun(fout, "attach", x, y); }
-	
+    else if ( isSigEnable(sig, x, y) )				{ printfun(fout, "enable", x, y); }
+    else if ( isSigControl(sig, x, y) )				{ printfun(fout, "control", x, y); }
+
 	else {
-        cerr << "NOT A SIGNAL : " << *sig << endl;
-        //exit(1);
+        stringstream error;
+        error << "Not a signal : " << *sig << endl;
+        throw faustexception(error.str());
 	}
 	return fout;
 }
