@@ -22,11 +22,12 @@
 #include    <stdlib.h>
 #include    <stdio.h>
 #include    <string.h>
-#include    "symbol.hh"
-#include    "compatibility.hh"
 #include    <iostream>
 #include    <cstring>
-#include    <assert.h>
+
+#include    "exception.hh"
+#include    "symbol.hh"
+#include    "compatibility.hh"
 
 using namespace std;
 
@@ -36,6 +37,7 @@ using namespace std;
  
 Symbol*	Symbol::gSymbolTable[kHashTableSize];
 
+map<const char*, unsigned int> Symbol::gPrefixCounters;
 
 /**
  * Search the hash table for the symbol of name \p str or returns a new one.
@@ -45,37 +47,33 @@ Symbol*	Symbol::gSymbolTable[kHashTableSize];
   
 Symbol* Symbol::get(const string& str)
 {
-	char 	buf[1024];
-	int		i;
-	int		n = str.length();
-	
-	if (n>1023) n = 1023;
-	for (i = 0; i < n; i++) { buf[i] = str[i]; }
-	buf[i] = 0;
-	
-	return Symbol::get(buf);
+    return Symbol::get(str.c_str());
 }
 
-
-
 /**
  * Search the hash table for the symbol of name \p str or returns a new one.
  * \param str the name of the symbol
  * \return a symbol of name str
  */ 
 
-Symbol* Symbol::get(const char* str)
+Symbol* Symbol::get(const char* rawstr)
 {
-    unsigned int 			hsh  = calcHashKey(str);
+    // ---replaces control characters with white spaces---
+    string str = rawstr;
+    for (size_t i = 0; i < str.size(); i++) {
+        char c = rawstr[i];
+        str[i] = (c >= 0 && c < 32) ? 32 : c;
+    }
+    unsigned int 	hsh  = calcHashKey(str.c_str());
     int 			bckt = hsh % kHashTableSize;
 	Symbol*			item = gSymbolTable[bckt];
-
-    while ( item && !item->equiv(hsh,str) ) item = item->fNext;
+  
+    while (item && !item->equiv(hsh, str.c_str())) item = item->fNext;
 	Symbol* r = item ? item : gSymbolTable[bckt] = new Symbol(str, hsh, gSymbolTable[bckt]);
+     
 	return r;
 }
 
-
 /**
  * Static method that searches the symbol table for a string. 
  * \param str string to search 
@@ -84,7 +82,7 @@ Symbol* Symbol::get(const char* str)
 
 bool Symbol::isnew(const char* str)
 {
-    unsigned int 			hsh  = calcHashKey(str);
+    unsigned int    hsh  = calcHashKey(str);
     int 			bckt = hsh % kHashTableSize;
 	Symbol*			item = gSymbolTable[bckt];
 	
@@ -92,7 +90,6 @@ bool Symbol::isnew(const char* str)
 	return item == 0;
 }
 
-
 /**
  * Creates a new symbol with a name obtained by concatenating the \p str prefix with a number in order to make it unique
  * \param str the prefix of the name
@@ -101,19 +98,16 @@ bool Symbol::isnew(const char* str)
 
 Symbol* Symbol::prefix (const char* str)
 {
-	char 	name[256];
-    
-    static map<const char*, unsigned int> gPrefixCounters;
-	
+	char name[256];
+  	
 	for (int n = 0; n<10000; n++) {
 		snprintf(name, 256, "%s%d", str, gPrefixCounters[str]++);
 		if (isnew(name)) return get(name);
 	}
-	assert(false);
+	faustassert(false);
 	return get("UNIQUEOVERFLOW");
 }	
 
-
 /**
  * Check if the name of the symbol is equal to string \p str
  * This method is used by isnew() and make() when searching the hashtable 
@@ -126,11 +120,9 @@ Symbol* Symbol::prefix (const char* str)
  
 bool Symbol::equiv (unsigned int hash, const char *str) const
 {
-	return (fHash == hash) && (strcmp(fName,str) == 0);
+	return (fHash == hash) && (strcmp(fName.c_str(), str) == 0);
 }
 
-
-
 /**
  * Compute the 32-bits hash key of string \p str
  * \param str the string
@@ -140,38 +132,37 @@ bool Symbol::equiv (unsigned int hash, const char *str) const
 unsigned int Symbol::calcHashKey (const char* str)
 {
     unsigned int h = 0;
-
+    
     while (*str) h = (h << 1) ^ (h >> 20) ^ (*str++);
     return h;
 }
 
-
-
 /**
  * Constructs a symbol ready to be placed in the hash table. 
- * It makes a private copy of its name.
+ * Gets a string to be kept.
  * \param str the name of the symbol
  * \param hsh the hash key of the symbol
  * \param nxt a pointer to the next symbol in the hash table entry
  */
 
-Symbol::Symbol(const char* str, unsigned int hsh, Symbol* nxt)
+Symbol::Symbol(const string& str, unsigned int hsh, Symbol* nxt)
 {
-	int len = strlen(str);
-	
-    fName = new char [len+1];
-    memcpy(fName, str, len+1);
+    fName = str;
     fHash = hsh;
     fNext = nxt;
 	fData = 0;
 }
 
 Symbol::~Symbol ()
-{
-	delete [] fName;
-}
+{}
 
 ostream& Symbol::print (ostream& fout) const 					///< print a symbol on a stream
 {
 	return fout << fName;
 }
+
+void Symbol::init ()
+{
+    gPrefixCounters.clear();
+    memset(gSymbolTable, 0, sizeof(Symbol*) * kHashTableSize);
+}
